/**
 * \file mgui/formchecklist.h
 * \brief MGUI::FORM_CHECKLIST definitions.
 *
 * \if NODOC
 * $Id: formchecklist.h_v 1.14 2011/11/22 21:37:32 mju Exp $
 *
 * $Log: formchecklist.h_v $
 * Revision 1.14  2011/11/22 21:37:32  mju
 * Fix dox dependencies.
 *
 * Revision 1.13  2010/09/27 16:16:27  mju
 * Make setEnabled/Visible public.
 *
 * Revision 1.12  2010/09/27 16:12:10  mju
 * Add setEnabled/Visible overrides.
 *
 * Revision 1.11  2010/05/14 21:43:09  mju
 * Support multi-column lists.
 *
 * Revision 1.10  2010/05/14 16:25:25  mju
 * Redesign to not expose implementation and use consistent method naming.
 *
 * Revision 1.9  2008/09/26 21:28:37  bcolombe
 * Removed other constructor
 *
 * Revision 1.8  2008/09/26 17:41:21  bcolombe
 * Added constructor with gridstyle and sizealign
 *
 * Revision 1.1  2006/11/15 20:54:29  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMCHECKLIST_H
#define	INC_MGUI_FORMCHECKLIST_H

#ifndef INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class BITSET;
namespace MGUI {
	class LAYOUT_PANE_BASE;
	}
#endif


namespace MGUI {

//===================================================================================================================

//! Form to show list of items with checkbox in front of each.
class FORM_CHECKLIST : public MGUI::FORM_COMPOSITE {
	public:
	
		enum STYLES {
			STYLE_Default = 				0x00,	//!< Default style, no selection rules
			STYLE_FirstIsExclusive =	0x01,	//!< First item is exclusive of others, toggling it on will toggle others off automatically
			STYLE_AtLeastOne =			0x02,	//!< Must have at least one item selected
			STYLE_OnlyOne =				0x04,	//!< There can be only one selected
			STYLE_ShowScrollbar =		0x08,	//!< Show vertical scrollbar
			STYLE_ShowFrame =				0x10,	//!< Show frame around list
			};

		//! Constructor.
		FORM_CHECKLIST ();

		//! Destructor.
		virtual ~FORM_CHECKLIST ();

		//! Add single item to end of list.
		//! For best results, add items before calling Create() to allow automatic sizing.
		//! No selection validation is done for items added after Create(), programmer is responsible for setting to conform to control STYLE.
		//! @return Zero-based index of item added.
		UINT16 AddItem (
			ID id,											//!< ID to attach to item
			const MISTRING& label,						//!< String to show in list
			bool selected = false						//!< Set item as 'selected'
			);

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int rows,										//!< Height in rows
			int columns = 1,								//!< Number of list columns
			STYLES Style = STYLE_Default,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create with column heading.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& ColumnLabel,				//!< Label for the string column
			int rows,										//!< Height in rows
			int columns = 1,								//!< Number of list columns
			STYLES Style = STYLE_Default,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Delete all items from control.
		void DeleteAllItems (
			);

		//! Delete item from list given index.
		void DeleteItem (
			UINT16 index									//!< Zero-based index of item to delete
			);

		//! Delete item from list given ID.
		void DeleteItemID (
			ID id												//!< ID of item to delete
			);

		//! Retrieve number of items in the list.
		UINT16 GetCount (
			) const;

		//! Get index to item from ID.
		//! @return Zero-based index or UINT16_MAX if no matching item found.
		UINT16 GetIndexFromID (
			ID id												//!< ID of item to retrieve index of
			) const;

		//! Get ID associated with list item.
		//! @return ID specified when item was added.
		ID GetItemID (
			UINT16 index									//!< Zero-based index of item
			) const;

		//! Get text string associated with list item.
		//! @return String containing text, will be empty if invalid item.
		MISTRING GetItemString (
			UINT16 index									//!< Zero-based index of item
			) const;

		//! Get number of selected items.
		//! @return Number of selected items or 0 if none selected.
		UINT16 GetSelectedCount (
			) const;
			
		//! Retrieve set of selected items.
		const BITSET& GetSelectedSet (
			) const;

		//! Insert item at specified position.
		//! For best results, add items before calling Create() to allow automatic sizing.
		//! No selection validation is done for items added after Create(), programmer is responsible for setting to conform to control STYLE.
		//! @return Zero-based index of item added
		UINT16 InsertItem (
			UINT16 index,									//!< Zero-based index of position at which item should be inserted
			ID id,											//!< ID to attach to item
			const MISTRING& string,						//!< String to show in list
			bool selected = false						//!< Set item as 'selected'
			);

		//! Determine if specified item is selected.
		bool IsItemSelected (
			UINT16 index									//!< Zero-based index of item
			) const;

		//! Determine if specified item is selected.
		bool IsItemSelectedID (
			ID id												//!< ID of item to retrieve index of
			) const;

		//! Set all items as selected or not.
		void SetAllSelected (
			bool selected,
			bool notify = true
			);

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Select whether item is selected using index.
		//! @return 'True' if state was changed, 'false' if not.
		bool SetItemSelected (
			UINT16 index,									//!< Zero-based index of item
			bool selected,
			bool notify = true
			);

		//! Select whether item is selected using ID.
		//! @return 'True' if state was changed, 'false' if not.
		bool SetItemSelectedID (
			ID id,											//!< ID of item to select
			bool selected,
			bool notify = true
			);

		//! Resize control to be able to show widest item.
		void SizeToContentWidth (
			);

		//! Set whether form is enabled.
		virtual void SetEnabled (bool enabled);

		//! Set whether form is visible.
		virtual void SetVisible (bool visible);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		// Unimplemented
		FORM_CHECKLIST (const FORM_CHECKLIST&);
		FORM_CHECKLIST& operator= (const FORM_CHECKLIST&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_CHECKLIST::STYLES);
#endif

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMCHECKLIST_H
