/**
 * \file colorquant.h <mi32/colorquant.h>
 * \brief Color quantization class for converting RGB/RGBA images to palette-color images
 *
 * \if NODOC
 * $Id: colorquant.h_v 1.6 2010/11/23 18:33:11 bcolombe Exp $
 *
 * $Log: colorquant.h_v $
 * Revision 1.6  2010/11/23 18:33:11  bcolombe
 * Add ShowStatus
 *
 * Revision 1.5  2010/06/07 13:31:23  mju
 * Don't make v_getSourceMaskRow and v_getSourceSampleRow pure.
 *
 * Revision 1.4  2010/06/03 15:40:16  bcolombe
 * Added GetSourceMaskRow, GetSourceSampleRow, IsSampleContiguous
 *
 * Revision 1.3  2008/05/12 19:22:00  mju
 * Add getMaxColors.
 *
 * Revision 1.2  2007/01/19 16:09:19  mju
 * Use color_array.
 *
 * Revision 1.1  2005/05/20 17:58:44  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COLORQUANT_H
#define	INC_MI32_COLORQUANT_H

#ifndef	INC_MI32_COLORSPC_H
#include <mi32/colorspc.h>
#endif

class COLOR_ARRAY;


class COLORQUANTIZER {
	public:

		enum DITHER {
			DITHER_None,
			DITHER_FloydSteinberg
//			DITHER_Ordered
			};

		enum REPRESENTATIVE {
			REPRESENTATIVE_Center,
			REPRESENTATIVE_AverageColor,
			REPRESENTATIVE_AveragePixel
			};

		COLORQUANTIZER ();

		virtual ~COLORQUANTIZER ();

		//! Compute color palette only.
		ERRVALUE ComputePalette (
			COLOR_ARRAY& Palette			//!< Palette returned
			);

		//! Compute palette and perform conversion.
		ERRVALUE Convert (
			);

		//! Compute palette and perform conversion.
		ERRVALUE Convert (
			const COLOR_ARRAY& Palette		//!< Palette used
			);

		//! Get maximum number of output colors.
		int GetMaxColors (
			) const { return (m_MaxColors); }

		//! Set dither method.
		//! Default is Floyd-Steinberg if not previously set.
		void SetDither (
			DITHER dither
			) { m_dither = dither; }

		//! Set maximum number of palette colors to generate.
		//! Default is 256 if not set.
		void SetMaxColors (
			int MaxColors								//!< Maximum number of palette colors to generate
			) { if (MaxColors > 1 && MaxColors <= 256) m_MaxColors = MaxColors; }

		//! Set maximum precision for representing color.
		//! Default precision is 8 bits per component if not set.
		//! When determining the color usage, a count of each unique RGBA value is determined, up to a maximum
		//! of 32K unique values.  If the number of unique values exceeds this maximum the precision is reduced
		//! and the source image is scanned again.  If it is known in advance that the number of unique values
		//! may be large, time may be saved by starting with a lower precision value.
		//! Recommendation is to use 6 or 7 bits if source based on a pictorial image (eg photo or satellite)
		//! and 8 bits if source is based on a graphical drawing (eg vector, shape, etc).
		void SetMaxPrecision (
			int MaxPrecision							//!< Maximum precision per component, must be between 4 and 8
			) { m_MaxPrecision = MaxPrecision; }

		//! Set method for determining representative color.
		//! Default is AverageColor if not set.
		void SetRepresentative (
			REPRESENTATIVE representative
			) { m_representative = representative; }

		//! Set whether to show status context updates
		void SetShowStatus (
			bool bShowStatus
			) { m_bShowStatus = bShowStatus; }

		// Get pointer to source image row.
		ERRVALUE GetSourceRow (
			INT32 row,									//!< Image row number
			const void *& pImageRow,				//!< Pointer to start of image row returned
			const UINT8 *& pMaskRow					//!< Pointer to start of mask row returned, NULL if no mask
			) { return (v_GetSourceRow(row,pImageRow,pMaskRow)); }

		// Get pointer to source image row.
		ERRVALUE GetSourceSampleRow (
			INT32 row,								//!< Image row number
			UINT16 SampleNum,						//!< Sample number
			const UINT8 *& pImageRow				//!< Pointer to start of image row returned
			) { return (v_GetSourceSampleRow(row,SampleNum,pImageRow)); }

		// Get pointer to source mask row.
		ERRVALUE GetSourceMaskRow (
			INT32 row,								//!< Image row number
			const UINT8 *& pMaskRow					//!< Pointer to start of mask row returned, NULL if no mask
			) { return (v_GetSourceMaskRow(row,pMaskRow)); }

		//! Called to check if samples are contiguous
		bool IsSampleContiguous (
			) { return (v_IsSampleContiguous()); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DITHER m_dither;
		REPRESENTATIVE m_representative;
		int m_MaxColors;
		int m_MaxPrecision;
		bool m_bShowStatus;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Called at end of each target image row.
		//! Default implementation does nothing.
		virtual ERRVALUE v_EndTargetRow (
			INT32 row									//!< Image row number
			);

		//! Get number of columns in image.
		virtual INT32 v_GetNumColumns (
			) = 0;

		//! Get number of rows in image.
		virtual INT32 v_GetNumRows (
			) = 0;

		//! Get source image colorspace.
		//! Must return one of RGB or RGBA colorspace variants in any supported ordering.
		virtual COLORSPACE v_GetSourceColorspace (
			) = 0;

		//! Called to get pointers to source image and mask rows.
		virtual ERRVALUE v_GetSourceRow (
			INT32 row,									//!< Image row number
			const void *& pImageRow,				//!< Pointer to start of image row returned
			const UINT8 *& pMaskRow					//!< Pointer to start of mask row returned, return NULL if no mask
			) = 0;

		//! Called to get pointers to source mask rows.
		virtual ERRVALUE v_GetSourceMaskRow (
			INT32 row,								//!< Image row number
			const UINT8 *& pMaskRow				//!< Pointer to start of mask row returned, return NULL if no mask
			);

		//! Called to get pointers to source image and mask rows.
		virtual ERRVALUE v_GetSourceSampleRow (
			INT32 row,								//!< Image row number
			UINT16 SampleNum,						//!< Sample number
			const UINT8 *& pImageRow			//!< Pointer to start of image row returned
			);

		//! Called to check if samples are contiguous.  
		//! Default implementation returns false.
		virtual bool v_IsSampleContiguous (
			);

		//! Called to get pointer to target image row.
		//! Default implementation returns EProgramError.
		virtual ERRVALUE v_GetTargetRow (
			INT32 row,									//!< Image row number
			UINT8 *& pImageRow						//!< Pointer to start of image row returned
			);

		//! Called when target palette has been computed.
		//! Override may modify palette as needed.
		//! Default implementation returns EProgramError.
		virtual ERRVALUE v_PutTargetPalette (
			COLOR_ARRAY& palette			//!< Palette passed/returned
			);
	};

#endif	/// INC_MI32_COLORQUANT_H
