/****
 *	\file mi32/filetext.h
 * \brief Definitions for FILE_TEXT class.
 *
 * \if NODOC
 * $Id: filetext.h_v 1.9 2011/10/13 12:44:20 mju Exp $
 *
 * $Log: filetext.h_v $
 * Revision 1.9  2011/10/13 12:44:20  mju
 * Remove redundant hdrs.
 *
 * Revision 1.8  2007/01/12 16:45:00  dwilliss
 * Changed default encoding to UTF8, not ASCII
 *
 * Revision 1.6  2005/03/04 23:35:53  dfriberg
 * Detect new line type if possible
 *
 * Revision 1.1  2004/10/18 16:29:17  dfriberg
 * Initial revision
 *
 * \endif
****/

#ifndef	INC_MI32_FILETEXT_H
#define	INC_MI32_FILETEXT_H

#ifndef  INC_MI32_GENRCRWTXT_H
#include <mi32/genrcrwtxt.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class FILEPATH;
class MISTRING;
class STRUTF8;
#endif

//! Class for encoded text file read/write.  Designed for simple text line-by-line reading,
//! full or partial text line writing, automatic handling of line terminations including
//! CR, LF, CR/LF, and LF/CR with possibility of mixing within a single file, and
//! character encoding conversion on read/write.  It should not be used for binary or mixed
//! text/binary files.
//!
//!	This class is not thread safe.  Most of the time the instance is specific to a 
//!		thread therefore the resource sharing issues are not involved.  If this class is
//!		a shared resource, it is up to the user of the class to guard it using the 
//!		appropriate locking method.
class CLASSLIBEXPORT FILE_TEXT : public GENERICRW_TEXT {
	public:
   
   #ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
   #endif

		//! Default constructor.
		FILE_TEXT (
			);

		//! Destructor, will close file if open.
		virtual ~FILE_TEXT (
			);

		//! Close file if presently open.
		void Close (
			);

      //! Get the approximate position of the file pointer.  Use for updating status bars.
      INT64 GetApproxFilePos() const;

		//! Determine if file is currently open.
		bool IsOpen (
			) const;
			
		//! Open specified file.
		//! If file is already open the previous file will be closed.
		ERRVALUE Open (
			const FILEPATH& filepath,							//!< Path to file to be opened or created
			FILE_ACCESS access = FILE_ACCESS_Read,			//!< Desired access
			CHAR_ENCODING encoding = CHAR_ENCODING_UTF8	//!< Encoding of file
			);

	private:

   #ifndef GENERATING_DOXYGEN_OUTPUT
      PRIV *m_pPriv;
   #endif

      //! Override to get current size of data.
      virtual INT64 v_GetSize() const;

      //! Override to read a line of text.
      virtual INT64 v_Read(void *buf, INT32 numbytes, bool AllowPartial);

      //! Override to write data.
      virtual ERRVALUE v_Write(const void *buffer, INT32 numbytes);

      //! Override to seek.
      virtual INT64 v_Seek(
         INT64 offset,
         GENERICRW::SEEKFROM from = GENERICRW::SEEKFROM_Beginning);

      //! Override to return current position.
      virtual INT64 v_Tell() const;

      //! Override to get capabilities.
      virtual GENERICRW_TEXT::CAPABILITIES v_GetCapabilities();

		FILE_TEXT (const FILE_TEXT&);
		FILE_TEXT& operator= (const FILE_TEXT&);
	};

#endif	// INC_MI32_FILETEXT_H

