/**
 * \file mi32/lidarlasrecord.h
 * \brief LIDAR::LAS::RECORD definitions
 *
 * \if NODOC
 * $Id: lidarlasrecord.h_v 1.5 2012/09/14 17:24:21 mju Exp $
 *
 * $Log: lidarlasrecord.h_v $
 * Revision 1.5  2012/09/14 17:24:21  mju
 * Add intensityMin/Max to filestats.
 * Add isFileCurrent to timestamp.
 *
 * Revision 1.4  2012/09/13 16:22:52  mju
 * Add filestats.get/setPointDensity.
 *
 * Revision 1.3  2012/09/12 20:22:39  mju
 * Add record_timestamp::setCurrent.
 *
 * Revision 1.2  2012/09/10 20:44:49  mju
 * Add accessor methods for record_filestats.
 *
 * Revision 1.1  2012/09/07 20:25:47  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LIDARLASRECORD_H
#define	INC_MI32_LIDARLASRECORD_H

#ifndef	INC_MI32_LIDARLASPOINT_H
#include <mi32/lidarlaspoint.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class DATETIME;
class FILEPATH;
class UNSIGNEDSET;
namespace LIDAR {
	namespace LAS {
		class FILE;
		}
	}
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


namespace LIDAR {
namespace LAS {

//=====================================================================================================================

//! Base for data records.
class CLASSLIBEXPORT RECORD {
	public:

		//! Constructor.
		RECORD ();

		//! Destructor.
		virtual ~RECORD ();

		//! Read record from file.
		//! @return TRUE if found, FALSE if not, < 0 if error
		int Read (
			LAS::FILE& file						//!< File to read from
			) { return (v_Read(file)); }

		//! Write record to file.
		ERRVALUE Write (
			LAS::FILE& file,						//!< File to write to
			bool ForceExtFile = false			//!< Force to use extended (LASX) file
			) const { return (v_Write(file,ForceExtFile)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		virtual int v_Read (LAS::FILE& file) = 0;
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const = 0;

		RECORD (const RECORD&);
		RECORD& operator= (const RECORD&);
		#endif
	};

//=====================================================================================================================

//! Timestamp record.
//! Used to determine if other records (RECORD_CLASSHISTOGRAM) are properly maintained.
class CLASSLIBEXPORT RECORD_TIMESTAMP : public RECORD {
	public:

		//! Default constructor, will be set to current date/time + 2 seconds.
		RECORD_TIMESTAMP ();

		//! Destructor.
		virtual ~RECORD_TIMESTAMP ();

		//! Get DATETIME.
		void GetDateTime (
			DATETIME& DateTime
			) const;

		//! Get date as YYYYMMDD.
		UINT32 GetDateYYYYMMDD () const
			{ return (m_DateYYYYMMDD); }

		//! Get time of day (seconds since midnight).
		UINT32 GetTimeOfDay () const
			{ return (m_TimeOfDay); }

		//! Determine if specified file is current relative to timestamp.
		//! A file is current if and only if its timestamp is no later than record timestamp.
		bool IsFileCurrent (
			const FILEPATH& filepath			//!< Path to file
			) const;

		//! Set to current date/time
		void SetCurrent ();

		//! Set from DATETIME.
		void SetDateTime (
			const DATETIME& DateTime
			);

		//! Set date/time.
		void SetDateTime (
			UINT32 DateYYYYMMDD,
			UINT32 TimeOfDay
			) { m_DateYYYYMMDD = DateYYYYMMDD; m_TimeOfDay = TimeOfDay; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_DateYYYYMMDD;
		UINT32 m_TimeOfDay;

		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;

		RECORD_TIMESTAMP (const RECORD_TIMESTAMP&);
		RECORD_TIMESTAMP& operator= (const RECORD_TIMESTAMP&);
		#endif
	};

//=====================================================================================================================

//! Base for "scan" records.
class CLASSLIBEXPORT RECORD_SCAN : public RECORD {
	public:

		//! Constructor.
		RECORD_SCAN ();

		//! Destructor.
		virtual ~RECORD_SCAN ();

		//! Add point to scan.
		ERRVALUE AddPoint (
			UINT64 PointIdx,
			const POINTRECORD& PointRec
			) { return (v_AddPoint(PointIdx,PointRec)); }

		//! Finalize after adding all points.
		ERRVALUE Finalize ()
			{ return (v_Finalize()); }

		//! Initialize scan values.
		ERRVALUE Initialize (
			LAS::FILE& file
			) { return (v_Initialize(file)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec) = 0;
		virtual ERRVALUE v_Initialize (LAS::FILE& file) = 0;
		virtual ERRVALUE v_Finalize ();

		RECORD_SCAN (const RECORD_SCAN&);
		RECORD_SCAN& operator= (const RECORD_SCAN&);
		#endif
	};

//=====================================================================================================================

//! Record for class histogram, with counts by class and flag combinations.
class CLASSLIBEXPORT RECORD_CLASSHISTOGRAM : public RECORD_SCAN {
	public:

		//! Constructor.
		RECORD_CLASSHISTOGRAM ();

		//! Destructor.
		virtual ~RECORD_CLASSHISTOGRAM ();

		//! Decrement counts based on point record.
		void Decrement (
			const POINTRECORD& PointRec
			);

		//! Get count for specified class index.
		UINT64 GetClassCount (
			UINT16 ClassIdx
			) const { return (m_ClassCount[ClassIdx]); }

		//! Get count for single CLASSFLAG value.
		UINT64 GetFlagCount (
			POINTRECORD::CLASSFLAGS flag		//!< Single CLASSFLAG value
			) const;

		//! Get count for specific CLASSFLAGS combination.
		UINT64 GetFlagsCount (
			POINTRECORD::CLASSFLAGS flags		//!< Single CLASSFLAG value
			) const { return (m_FlagsCount[flags]); }

		//! Increment counts based on point record.
		void Increment (
			const POINTRECORD& PointRec
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT64 m_ClassCount[256];
		UINT64 m_FlagsCount[16];

		void Clear ();

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_CLASSHISTOGRAM (const RECORD_CLASSHISTOGRAM&);
		RECORD_CLASSHISTOGRAM& operator= (const RECORD_CLASSHISTOGRAM&);
		#endif
	};

//=====================================================================================================================

//! Record for statistics about entire LAS file.
class CLASSLIBEXPORT RECORD_FILESTATS : public RECORD_SCAN {
	public:

		//! Constructor.
		RECORD_FILESTATS ();

		//! Destructor.
		virtual ~RECORD_FILESTATS ();

		const LRECT3D& GetExtentsRaw () const
			{ return (m_ExtentsRaw); }

		UINT16 GetIntensityMin () const
			{ return (m_IntensityMin); }

		UINT16 GetIntensityMax () const
			{ return (m_IntensityMax); }

		UINT64 GetNumByReturnNumber (UINT8 n) const
			{ return (m_ReturnNumberCount[n]); }

		UINT64 GetNumByReturnType (UINT8 t) const
			{ return (m_ReturnTypeCount[t]); }

		UINT64 GetNumEndOfSweep () const
			{ return (m_NumEndOfSweep); }

		UINT64 GetNumOutsideHdrExtents () const
			{ return (m_NumOutsideHdrExtent); }

		float GetPointDensity () const
			{ return (m_PointDensity); }

		const FLOAT_RANGE& GetRangeScanAngle () const
			{ return (m_RangeScanAngle); }

		const DOUBLE_RANGE& GetRangeTime () const
			{ return (m_RangeTime); }

		const FLOAT_RANGE& GetRangeWaveOffset () const
			{ return (m_RangeWaveOffset); }

		const FLOAT_RANGE& GetRangeWaveXt () const
			{ return (m_RangeWaveXt); }

		const FLOAT_RANGE& GetRangeWaveYt () const
			{ return (m_RangeWaveYt); }

		const FLOAT_RANGE& GetRangeWaveZt () const
			{ return (m_RangeWaveZt); }

		bool HasSourceIDs () const
			{ return (m_HasSourceIDs); }

		void SetPointDensity (
			float PointDensity
			) { m_PointDensity = PointDensity; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LRECT3D m_ExtentsHdrRaw;
		LRECT3D m_ExtentsRaw;
		UINT64 m_ReturnNumberCount[16];
		UINT64 m_ReturnTypeCount[6];
		UINT64 m_NumEndOfSweep;
		UINT64 m_NumOutsideHdrExtent;
		FLOAT_RANGE m_RangeScanAngle;
		DOUBLE_RANGE m_RangeTime;
		FLOAT_RANGE m_RangeWaveOffset;
		FLOAT_RANGE m_RangeWaveXt;
		FLOAT_RANGE m_RangeWaveYt;
		FLOAT_RANGE m_RangeWaveZt;
		float m_PointDensity;
		UINT16 m_IntensityMin;
		UINT16 m_IntensityMax;
		bool m_HasSourceIDs;

		void Clear ();

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_FILESTATS (const RECORD_FILESTATS&);
		RECORD_FILESTATS& operator= (const RECORD_FILESTATS&);
		#endif
	};

//=====================================================================================================================

//! Record for set of "bad points".
//! Points are considered "bad" if they lie outside the file header extents or have invalid "return" values.
//! Only written to file if bad points exist.
class CLASSLIBEXPORT RECORD_BADPOINTSET : public RECORD_SCAN {
	public:

		//! Constructor.
		RECORD_BADPOINTSET ();

		//! Destructor.
		virtual ~RECORD_BADPOINTSET ();

		//! Get set.
		const UNSIGNEDSET& GetSet () const
			{ return (*m_pSet); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LRECT3D m_ExtentsHdrRaw;
		UNSIGNEDSET *m_pSet;

		// RECORD overrides.
		virtual int v_Read (LAS::FILE& file);
		virtual ERRVALUE v_Write (LAS::FILE& file, bool ForceExtFile) const;
		// RECORD_SCAN overrides.
		virtual ERRVALUE v_AddPoint (UINT64 PointIdx, const POINTRECORD& PointRec);
		virtual ERRVALUE v_Initialize (LAS::FILE& file);

		RECORD_BADPOINTSET (const RECORD_BADPOINTSET&);
		RECORD_BADPOINTSET& operator= (const RECORD_BADPOINTSET&);
		#endif
	};

//=====================================================================================================================

/*
class CLASSLIBEXPORT WAVEFORMPACKETDESC {
	public:

		//! Constructor.
		RECORD_WAVEFORMPACKETDESC ();

		//! Destructor.
		virtual ~RECORD_WAVEFORMPACKETDESC ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT8 m_BitsPerSample;
		UINT8 m_CompressionType;
		UINT32 m_NumSamples;
		UINT32 m_TemporalSampleSpacing;
		double m_DigitizerGain;
		double m_DigitizerOffset;

		RECORD_WAVEFORMPACKETDESC (const RECORD_WAVEFORMPACKETDESC&);
		RECORD_WAVEFORMPACKETDESC& operator= (const RECORD_WAVEFORMPACKETDESC&);
		#endif
	};
*/

//=====================================================================================================================

}	// End namespace LAS
}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARLASRECORD_H
