/******************************************************************************
 * \file mi32/mg2mem32.h
 * \brief MGD::DEVICE_MEM32 implementation.
 *
 * \if NODOC
 * $Log: mg2mem32.h_v $
 * Revision 1.18  2012/02/20 20:26:50  mju
 * Add getImagePointerConst and allow bitwise ops on format.
 *
 * Revision 1.17  2012/02/20 18:39:35  mju
 * Move some methods to derived device_mem32rgb and nest context.
 *
 * Revision 1.16  2011/10/28 14:47:00  mju
 * Add eraseFrameBuffer/s and resizeFrameBuffers.
 * Remove get/setFrameBufferPtr.
 *
 * Revision 1.15  2011/10/26 19:53:59  mju
 * Add findMinimalRectangle and get/setFrameBufferPtr methods.
 *
 * Revision 1.14  2011/10/26 17:58:01  mju
 * Frame buffer support.
 *
 * Revision 1.13  2008/04/02 20:58:28  mju
 * Make op= explicitly private.
 *
 * Revision 1.12  2006/01/17 14:39:08  mju
 * Add allowPad parm to create().
 *
 * Revision 1.10  2005/06/15 22:06:37  mju
 * Add v_eraseRectangle.
 *
 * Revision 1.8  2004/07/27 17:34:58  dwilliss
 * Added colorspace parameter to create
 *
 * Revision 1.3  2002/04/24 14:30:24  dwilliss
 * Don't override GetColorSpace() any more.  Base class implementation
 * determines it from the DevInfo
 *
 * Revision 1.1  2002/02/21 15:46:49  dwilliss
 * Initial revision
 * \endif
 */

#ifndef INC_MI32_MG2MEM32_H
#define INC_MI32_MG2MEM32_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGD {


//! Memory-based implementation of MGD::DEVICE for 32-bit visuals.
class DEVICE_MEM32 : public MGD::DEVICE {

	public:

		//! Memory format
		enum FORMAT {
			FORMAT_Default =			0x00,
			FORMAT_PadTo8 =			0x01,			//!< Pad rows to 8-byte boundary
			FORMAT_Invert =			0x02			//!< Invert rows in memory so first bottom row is first in buffer
			};

		//! constructor
		DEVICE_MEM32 ();

		//! Destructor
		virtual ~DEVICE_MEM32 ();

		//! Erase specified frame buffer, setting contents to 0's.
		void EraseFrameBuffer (
			UINT16 FrameBuffer					//!< Frame buffer index
			);

		//! Erase all frame buffers, setting contents to 0's.
		void EraseFrameBuffers ();

		//! Find minimal rectangle containing all pixels with non-zero alpha values in any frame buffer.
		void FindMinimalRectangle (
			LRECT2D& rectangle					//! Rectangle returned
			) const;

		//! Find minimal rectangle containing all pixels with non-zero alpha values in specified frame buffer.
		void FindMinimalRectangle (
			UINT16 FrameBuffer,					//! Frame buffer index
			LRECT2D& rectangle					//! Rectangle returned
			) const;

	protected:

		//! Context for 32-bit memory device.
		class CONTEXT : public MGD::CONTEXT {
			public:

				virtual ~CONTEXT ();

			protected:

				CONTEXT (
					MGD::DEVICE_MEM32 *pDevice
					);

				//! Erase rectangle to specified color, replacing pixels regardless of transparency.
				virtual ERRVALUE v_EraseRectangle (
					const LRECT2D& rectangle,				//!< Rectangle to erase
					const COLOR& color						//!< Color to erase with
					);

				virtual ERRVALUE v_PutImageSpanDirect (
					INT32 x,
					INT32 y,
					INT32 len,
					const void *data,
					const UINT8 *mask
					);

			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT
				// Unimplemented
				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);

				friend class MGD::DEVICE_MEM32;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Create frame buffers.
		ERRVALUE CreateFrameBuffers (
			UINT16 NumFrameBuffers = 1			//!< Number of frame buffers
			);

		//! Get memory format.
		FORMAT GetFormat () const
			{ return (m_Format); }

		//! Get number of memory bytes per row.
		UINT32 GetNumBytesPerRow () const
			{ return (m_BytesPerRow); }

		//! Resize all frame buffers to match current device dimensions.
		ERRVALUE ResizeFrameBuffers ();

		//! Set memory format.
		//! Must be done before creating frame buffers, typically in derived class Create method.
		void SetFormat (FORMAT format);

		virtual CAPABILITY v_GetCapabilities () const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<UINT8*> m_FrameBuffers;
		UINT32 m_BytesPerRow;
		FORMAT m_Format;

		// Unimplemented
		DEVICE_MEM32 (const DEVICE_MEM32&);
		DEVICE_MEM32& operator= (const DEVICE_MEM32&);

		const UINT8* GetImagePointerConst (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer) const;

		virtual ERRVALUE v_FillSpan (MGD::CONTEXT *gc, INT32 x1, INT32 x2, INT32 y);
		virtual UINT8* v_GetImagePointer (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer);
		virtual UINT16 v_GetNumFrameBuffers () const;

		friend class MGD::CONTEXT;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DEVICE_MEM32::FORMAT)
#endif


}	// End of namespace

#endif	// INC_MI32_MG2MEM32_H

