/******************************************************************************
 *
 * \file mi32/mg2mem32rgba.h
 * \brief A RAM-based implementation of MGD::DEVICE for 32-bit RGBA visuals
 *
 * \if NODOC
 *
 * $Id: mg2mem32rgba.h_v 1.15 2012/02/20 18:40:16 mju Exp $
 *
 * $Log: mg2mem32rgba.h_v $
 * Revision 1.15  2012/02/20 18:40:16  mju
 * Add format parm for create and nest context.
 *
 * Revision 1.14  2011/10/26 20:25:25  mju
 * Add create with multiple frame buffers.
 *
 * Revision 1.13  2011/10/26 19:54:20  mju
 * Moved findMinimalRectangle to base class.
 *
 * Revision 1.12  2011/10/26 17:58:04  mju
 * Frame buffer support.
 *
 * Revision 1.11  2010/05/05 14:35:11  mju
 * Add v_drawPixel override.
 *
 * Revision 1.10  2008/04/02 20:58:38  mju
 * Make op= explicitly private.
 *
 * Revision 1.9  2007/07/10 21:25:32  dwilliss
 * Added virtual method for what blend modes the device supports
 *
 * Revision 1.8  2006/01/17 14:39:15  mju
 * Add allowPad parm to create().
 *
 * Revision 1.7  2005/06/16 15:37:11  mju
 * Add v_Clear override.
 *
 * Revision 1.6  2005/04/22 17:38:03  mju
 * Add findMinimalRectangle.
 *
 * Revision 1.5  2005/01/26 17:45:52  mju
 * Remove scanColorUse.
 *
 * Revision 1.4  2004/07/27 17:35:14  dwilliss
 * Added colorspace parameter to create method
 *
 * Revision 1.3  2004/01/14 21:10:04  scowan
 * Added protected method.
 *
 * Revision 1.2  2003/12/15 23:36:51  dwilliss
 * Diverging for 7.0
 * Made changes necessary for graphical data tips
 *
 * Revision 1.1  2003/12/10 23:52:48  dwilliss
 * Initial revision
 *
 * \endif
 */

#ifndef INC_MI32_MG2MEM32RGBA_H
#define INC_MI32_MG2MEM32RGBA_H

#ifndef INC_MI32_MG2MEM32_H
#include <mi32/mg2mem32.h>
#endif


namespace MGD {

//! Memory-based drawing device for 32-bit RGBA visuals.
class DEVICE_MEM32RGBA : public MGD::DEVICE_MEM32 {

	public:

		//! constructor
		DEVICE_MEM32RGBA();

		//! Destructor
		virtual ~DEVICE_MEM32RGBA();

		//! Create device with single frame buffer.
		//! If the ColorSpace is provided, it must be one of the following:
		//!	COLORSPACE_A8_R8_G8_B8 (the default), 
		//!	COLORSPACE_R8_G8_B8_A8
		//!	COLORSPACE_A8_B8_G8_R8
		//!	COLORSPACE_B8_G8_R8_A8
		//! Any other colorspace will return an error
		ERRVALUE Create (
			INT32 height,							//!< Height (number of rows) in pixels
			INT32 width,							//!< Width (number of columns) in pixels
			COLORSPACE ColorSpace = COLORSPACE_A8_R8_G8_B8,
			FORMAT format = FORMAT_Default	//!< Memory format
			) { return (Create(height,width,1,ColorSpace,format)); }

		//! Create device with option for multiple frame buffers.
		//! If the ColorSpace is provided, it must be one of the following:
		//!	COLORSPACE_A8_R8_G8_B8 (the default), 
		//!	COLORSPACE_R8_G8_B8_A8
		//!	COLORSPACE_A8_B8_G8_R8
		//!	COLORSPACE_B8_G8_R8_A8
		//! Any other colorspace will return an error
		ERRVALUE Create (
			INT32 height,							//!< Height (number of rows) in pixels
			INT32 width,							//!< Width (number of columns) in pixels
			UINT16 NumFrameBuffers,				//!< Number of frame buffers
			COLORSPACE ColorSpace = COLORSPACE_A8_R8_G8_B8,
			FORMAT format = FORMAT_Default	//!< Memory format
			);

	protected:

		//! Context for 32-bit RGBA memory device.
		class CONTEXT : public MGD::DEVICE_MEM32::CONTEXT {
			public:

				virtual ~CONTEXT ();

			protected:

				CONTEXT (
					MGD::DEVICE_MEM32RGBA *pDevice
					);

				virtual ERRVALUE v_PutImageSpanDirect (
					INT32 x,
					INT32 y,
					INT32 len,
					const void *data,
					const UINT8 *mask
					);

			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT
				// Unimplemented.
				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);

				friend class MGD::DEVICE_MEM32RGBA;

				virtual ERRVALUE v_DrawPixel (INT32 x, INT32 y);

				#endif // GENERATING_DOXYGEN_OUTPUT
			};

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Unimplemented.
		DEVICE_MEM32RGBA (const DEVICE_MEM32RGBA&);
		DEVICE_MEM32RGBA& operator= (const DEVICE_MEM32RGBA&);

		void DrawSinglePixel (CONTEXT *gc, INT32 x, INT32 y);

		virtual ERRVALUE v_FillSpan (MGD::CONTEXT* vgc, INT32 x1, INT32 x2, INT32 y);
		virtual ERRVALUE v_Clear (const COLOR& color);
		virtual MGD::CONTEXT* v_CreateContext ();
		virtual void v_GetPixelColor (PIXEL pixel, COLOR& color) const;
		virtual PIXEL v_GetColorPixel (const COLOR& color);
		virtual bool v_SupportsOpacityBlendMode (MGD::OPACITY_MODE mode) const;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

		
}	//! End of namespace

#endif	// INC_MI32_MG2MEM32RGBA_H

