/**
 * \file mplayout.h <mi32/mplayout.h>
 * \brief Definitions for Map & Poster layout
 *
 * \if NODOC
 * $Id: mplayout.h_v 1.45 2012/01/06 17:52:17 mju Exp $
 *
 * $Log: mplayout.h_v $
 * Revision 1.45  2012/01/06 17:52:17  mju
 * Remove redundant hdr.
 *
 * Revision 1.44  2011/10/07 15:10:59  mju
 * Dox.
 *
 * Revision 1.42  2007/07/18 22:09:55  dwilliss
 * added flag for anaglyph printing
 *
 * Revision 1.41  2007/04/09 17:48:20  dwilliss
 * Changed printer gamma structure
 *
 * Revision 1.39  2006/09/15 15:26:38  mju
 * Serializer chagnes.
 *
 * Revision 1.35  2003/06/16 22:56:13  dwilliss
 * Added Serlialzer itemdef method
 *
 * Revision 1.31  2003/05/28 15:06:38  dwilliss
 * Added ICM profile members
 *
 * Revision 1.28  2003/04/29 14:28:26  dwilliss
 * Added some dither pattern parameters to LAYOUTPARMS
 *
 * Revision 1.27  2002/08/14 22:41:33  dwilliss
 * Paper size is now Unicode
 *
 * Revision 1.22  1999/02/03  22:48:03  dwilliss
 * Added FullPage flag
 *
 * Revision 1.10  1997/09/24  23:04:58  dwilliss
 * Added Zoom to LAYOUTPARMS
 *
 * Revision 1.9  1997/09/16  17:21:18  coffee!dwilliss
 * Increased size of printer and paper size strings
 *
 * Revision 1.7  1997/08/07  16:04:59  dwilliss
 * Prototype MprintLayoutSetup
 *
 * Revision 1.4  1997/08/04  16:17:15  dwilliss
 * Added proto for MprintTestStripDialog()
 * Made prototype for MprintPageSetup() more strict if
 * INC_MI32_MDISP_H is defined.  Old code may get proto
 * from dispwind.h anyway.
 *
 * Revision 1.1  1997/05/02  14:35:09  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MPLAYOUT_H
#define	INC_MI32_MPLAYOUT_H

#ifndef INC_MI32_PRINTGAMMA_H
	#include <mi32/printgamma.h>		
#endif

#ifndef INC_MI32_COLORBAL_H
	#include <mi32/colorbal.h>		// for defn of CBPARMS
#endif

#ifndef INC_MI32_MG2CMS_H
	#include <mi32/mg2cms.h>		// for defn of PROFILE
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declaration
namespace GRE {
	class LAYOUT;
	}
#endif


//! Hardcopy layout parameters.
struct LAYOUTPARMS {

	LAYOUTPARMS();
	~LAYOUTPARMS();
	LAYOUTPARMS(const LAYOUTPARMS&);

	LAYOUTPARMS& operator=(const LAYOUTPARMS&);

	static const SERIALIZERITEM* GetSerialItemDef();

	//! Read LAYOUTPARMS from Ini file.
	ERRVALUE ReadDefault();

	//! Write LAYOUTPARMS to Ini file.
	ERRVALUE WriteDefault();

	double FullHeight;						//!< Full height including margins in inches
	double FullWidth;							//!< Full width including margins in inches
	double FullPrintableHeight;			//!< Full printable height (user selected) in inches
	double FullPrintableWidth;				//!< Full printable width (user selected) in inches
	double PrintableHeight;					//!< Printable height of a single page in inches (fixed)
	double PrintableWidth;					//!< Printable width of a single page in inches (fixed)
	double LeftMargin;						//!< Unprintable left margin in inches (fixed)
	double TopMargin;							//!< Unprintable top margin in inches (fixed)
	double Scale;								//!< eg: 24000 for 1:24000 (Should have been named MapScale)
	double Zoom;								//!< Zoom factor (normally 1.0)
	double dpi;									//!< Printer resolution in dots/inch
	INT32	 flags;								//!< misc flags, see below
	char 	 printer[64];	//!< Name of driver object.  Not localizable.
	MIUNICODE	 paper[64];
	char   rpattern[32];
	char   vpattern[32];
	double contrast;
	double brightness;
	CBPARMS	cbparms;							//!< Color balance parameters
	PRINTGAMMA	gamma[3];
	double DotGain[4];		//!< Percent (0.0 to 1.0)
	double ScreenAngle[4];	//!< in degrees
	double ScreenFreq;		//!< in LPI. All componants must be same frequency
	MGD::PROFILEPARMS_PRINTER ProfileParms;		
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
PREVENT_MEMFUNC(LAYOUTPARMS)
#endif

#define MAPLAYOUT_SetupDone	(0x00000001)
#define MAPLAYOUT_UserCBal 	(0x00000008)
#define MAPLAYOUT_UserGamma 	(0x00000010)
#define MAPLAYOUT_Landscape	(0x00000200)	//! Sizes in LAYOUTPARMS are rotated 90 degrees from actual paper
#define MAPLAYOUT_ScaleToPage	(0x00000400)	//! Adjust Zoom to make the layout fit the page
#define MAPLAYOUT_Anaglyph		(0x00001000)	//! Print anaglyph stereo

//----------------------------------------------------------------------------
//		Flags for MprintPageSetup															
//----------------------------------------------------------------------------

#define PRSETUP_FromRast	0x00000001	//!< For Support / Print From / Print Raster...
#define PRSETUP_PRF			0x00000002	//!< For Support / Print From / Print File...
#define PRSETUP_NoPrint		0x00000004	//!< changes "print" button to "ok"
#define PRSETUP_NoManage	0x00000008	//!< Check settings, but don't popup unless problem
#define PRSETUP_Snapshot	0x00000010	//!< Printing a snapshot

//----------------------------------------------------------------------------
//		Functions and Macros																		
//----------------------------------------------------------------------------

#ifndef GENERATING_DOXYGEN_OUTPUT
struct PRINTERDRIVER;
struct PRINTEROPTIONS;
#endif

extern "C" {

//! Read LAYOUTPARMS from Ini file.
int MxReadDefaultLayout (
	LAYOUTPARMS *parms
	);

//! Write LAYOUTPARMS to Ini file.
int MxWriteDefaultLayout (
	LAYOUTPARMS *parms
	);

//!	Print test strip dialog.
//! 
//! Note.  Except for the 3rd parm which changes from a DWWINDOW to an MDISPLAYOUT 
//! The API of this function cannot change until we eliminate printrvc.
//!
//!	This function gets passed to MprintSetup as it's 3rd parameter.  It's assumed
//!	that the 4th parameter to MprintSetup (the HookData) is really an MDISPLAYOUT
//! and that that gets passed to us as our 3rd parameter.
//!
//!	If "parent" is not NULL, then it will create a non-modal dialog for keeping 
//! track of test strip data.  The main form widget for this dialog is returned
//! in *RetWidget.
//!	If parent is NULL, it just looks at the mode parameter and changes the formatting
//!	of its list widget to match the given mode.  
//!
//!	It is also assumed that the CBPARMS and PRINTGAMMA passed to the function
//!	will stay in the same memory location for the life of the dialog (which
//!	is non-modal).  When the user clicks the "add" button on the dialog, it
//!	makes copies of these structures.
//!
//! The cbfunc is called whenever the user selects a line from the list widget
//!	of test settings.  cbfunc will be called and given the CBPARMS and PRINTGAMMA
//!	that were saved for that test.  It's up to cbfunc to adjust the color balance
//!	controls to show those settings.
//!
int MprintTestStripDialog (
	Widget parent, 
	Widget *RetWidget, 
	GRE::LAYOUT *layout, 
	CBPARMS *cbal, 
	PRINTGAMMA *cont, 
	void (*cbfunc)(void *, CBPARMS *, PRINTGAMMA *), 
	void *cbdata, 
	PRINTERDRIVER *PrinterDriver,
	PRINTEROPTIONS *PrinterParms,
	int mode
	);

//! Print page setup.
int MprintPageSetup (
	Widget parent, 
	GRE::LAYOUT *layout,
	int (*TestStripFunc)(Widget, Widget*, GRE::LAYOUT*, CBPARMS*,PRINTGAMMA*, 
		void (*func)(void*, CBPARMS*, PRINTGAMMA*),void*, PRINTERDRIVER *PrinterDriver,
		PRINTEROPTIONS *PrinterParms, int) = 0,  //!< ie, MprintTestStripDialog
	UINT32 flags = 0
	);

//! Print layout setup.
int MprintLayoutSetup (
	Widget parent, 
	GRE::LAYOUT *layout, 
	UINT32 flags
	);

}	// extern "C"

#endif	// INC_MI32_MPLAYOUT_H
