/**
 * \file mi32/polyline.h
 * \brief POLYLINE class definitions.
 *
 * \if NODOC
 * $Id: polyline.h_v 1.122 2012/11/09 18:04:11 mju Exp $
 *
 * $Log: polyline.h_v $
 * Revision 1.122  2012/11/09 18:04:11  mju
 * Remove legacy methods using coordop.
 *
 * Revision 1.121  2012/11/08 17:47:05  mju
 * Support spatref::coordtrans.
 *
 * Revision 1.120  2012/10/09 13:43:17  mju
 * Add round method.
 *
 * Revision 1.119  2012/05/18 17:10:29  mju
 * In convertToLongFloor with scale/offset use dpoint2d for scale.
 *
 * Revision 1.118  2012/05/18 16:52:13  mju
 * Add convertToLongRound with scale/offset.
 *
 * Revision 1.117  2012/04/04 17:31:18  mju
 * Add get/setVertexX/Y.
 *
 * Revision 1.116  2011/11/29 16:30:48  mju
 * Remove obsolete getExtentsOptimization.
 *
 * Revision 1.115  2011/11/07 16:44:49  mju
 * Add getExtentsIfKnown.
 *
 * Revision 1.114  2011/10/14 14:53:35  mju
 * Add exchange method.
 *
 * Revision 1.113  2011/10/10 16:33:32  mju
 * Add clip and validatePolygon using delegate.
 *
 * Revision 1.112  2011/10/07 13:07:17  mju
 * Remove unnecessary hdrs and add some dox.
 *
 * Revision 1.110  2010/11/03 19:02:45  mju
 * Remove triangulate method.
 *
 * Revision 1.107  2009/05/20 16:24:50  mju
 * Add computeInteriorPoint.
 *
 * Revision 1.106  2009/02/27 14:50:13  mju
 * Add excludeLineEndPoints to intersectSearchRaw.
 * Add intersectinfo::clean.
 *
 * Revision 1.105  2009/02/20 18:16:27  mju
 * In findIntersections return whether has common vertex.
 *
 * Revision 1.104  2008/10/17 14:35:46  mju
 * Add removeColinearVertices.
 *
 * Revision 1.102  2008/07/17 19:52:10  scowan
 * Added dist point method and changed percent point on line.
 *
 * Revision 1.95  2007/09/07 21:09:18  mju
 * Add removeDuplicateVertices (really does what the method says).
 *
 * Revision 1.88  2006/03/07 18:24:35  scowan
 * Added self line conflate method.
 *
 * Revision 1.87  2006/02/01 16:54:45  scowan
 * Added setSectionArray to polyline section class.
 *
 * Revision 1.82  2004/11/24 18:45:06  scowan
 * Major update of find intersections and combine methods.
 *
 * Revision 1.78  2004/10/27 21:13:41  scowan
 * Moved internal closest point searching from conflate to its own method.
 *
 * Revision 1.73  2004/08/17 21:30:20  mju
 * Change combineresult enum names to be clearer and not specific to 'clipping'.
 *
 * Revision 1.1  2001/01/04 22:54:20  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_POLYLINE_H
#define INC_MI32_POLYLINE_H

#ifndef	INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif


#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class BITSET_UNOWNED;
class MISTRING;
class POLYLINELIST;
class TRANS2D_AFFINE;
class TRANS2D_MAPGEN;
class TRANS3D;
class XMLNODE;
namespace SPATREF {
	class COORDTRANS;
	}
#endif


struct DTRIANGLE2D {
	DPOINT2D p1;
	DPOINT2D p2;
	DPOINT2D p3;
	};

enum CLIPMODE {
	CLIPMODE_Inside =		0,	//!< Clip to inside of region
	CLIPMODE_Outside =	1	//!< Clip to outside of region
	};

enum TESTCRITERIA {
	TESTCRITERIA_FullOutside = 0,  //!< Test if fully outside region
	TESTCRITERIA_PartOutside = 1,  //!< Test if any part is outside region
	TESTCRITERIA_FullInside = 2,	//!< Test if fully inside region
	TESTCRITERIA_PartInside = 3		//!< Test if any part is inside region
	};


//--------------------------------------------------------------------------------------------------------------------
//! The POLYLINE class encapsulates a 2D or 3D polyline.  Its primary use is for manipulation of
//! coordinates whether in 2D or 3D.  It does not maintain polygon topology (i.e. crossing segments are not
//! resolved).  The base storage method is a MIDOUBLEARRAY, by which alignment is maintained.
//!
//! A POLYLINE instance can have associated with it certain optimization setups to improve some of the methods
//! response times.  They are currently "Extents", "Section", and "Strip" optimizations.  The methods that can
//! make use of these optimizations will state that in the documentation for that method.
//--------------------------------------------------------------------------------------------------------------------

class GEOMLIBCLASSEXPORT POLYLINE {
	public:

		//! Enumeration of which side(s) of line to buffer.
		enum BUFFERSIDE {
			BUFFERSIDE_In =   1,
			BUFFERSIDE_Out =  2,
			BUFFERSIDE_Both = 3
			};

		//! Comparison result enumeration.
		enum COMPRESULT {
			COMPRESULT_Outside = 1,			//!< Both elements are outside of each other
			COMPRESULT_Inside,				//!< One element is completely inside the other
			COMPRESULT_Intersect,			//!< Both elements intersect
			COMPRESULT_Overlap				//!< One element overlaps the other
			};

		//! Values returned for Union(), Intersect(), Subtract() and ExclusiveUnion()
		enum COMBINERESULT {
			COMBINERESULT_Disjoint =		1,	//!< Polygons are disjoint
			COMBINERESULT_OperInsideSrc =	2,	//!< Operator polygon inside source polygon, may share border point(s)
			COMBINERESULT_SrcInsideOper =	3,	//!< Source polygon inside operator polygon, may share border point(s)
			COMBINERESULT_Equal =			4,	//!< Source polygon equals operator polygon
			COMBINERESULT_Intersect =		5,	//!< Source and operator polygons intersect
			// Legacy names
			COMBINERESULT_ClipOutside =	COMBINERESULT_Disjoint,
			COMBINERESULT_ClipInside =		COMBINERESULT_OperInsideSrc,
			COMBINERESULT_SrcInside =		COMBINERESULT_SrcInsideOper
			};

		enum EXTRACT {
			EXTRACT_StartPoint = 1,
			EXTRACT_CenterPoint,
			EXTRACT_EndPoint
			};
			
		enum JOINTYPE {
			JOINTYPE_Miter = 0,
			JOINTYPE_Round,
			JOINTYPE_Bevel
			};
			
		enum SPLINEFLAGS {
			SPLINEFLAG_None = 				0x0000,
			SPLINEFLAG_DontMoveEnds = 		0x0001,
			SPLINEFLAG_UseMinDistance = 	0x0002,
			SPLINEFLAG_TreatAsClosed = 	0x0004
			};
			
		enum SPLINE {
			SPLINE_NoSpline = 0,
			SPLINE_Cubic,
			SPLINE_Quadratic,
			SPLINE_Bezier
			};
			
		enum THINMETHOD {
			THINMETHOD_MinRatio = 1,
			THINMETHOD_Minimum,
			THINMETHOD_Douglas
			};

		class ITERATOR;

		//! Constant iterator.
		class CONST_ITERATOR {
			public:

				//! Default constructor
				CONST_ITERATOR (
					) :
					m_Index(0),
					m_PolyLine(0)
					{ }

				//! Internal constructor
				CONST_ITERATOR (
					INT32 index,
					const POLYLINE*const PolyLine
					) : 
					m_Index(index),
					m_PolyLine(PolyLine)
					{ }

				//! Copy constructor
				CONST_ITERATOR (
					const CONST_ITERATOR& rhs
					) : 
					m_Index(rhs.m_Index),
					m_PolyLine(rhs.m_PolyLine)
					{ }

				//! Copy constructor
				CONST_ITERATOR (
					const ITERATOR& rhs
					) : 
					m_Index(rhs.m_Index),
					m_PolyLine(rhs.m_PolyLine)
					{ }

				//! Assignment.
				CONST_ITERATOR& operator= (
					const CONST_ITERATOR& rhs
					) {
					if (this != &rhs) {
						m_Index = rhs.m_Index;
						m_PolyLine = rhs.m_PolyLine;
						}
					return (*this);
					}
					
				//! Dereference operator, returns reference to current vertex.
				const DPOINT2D& operator* (
					) const { return (m_PolyLine->GetVertex(m_Index)); }

				//! Dereference operator, returns pointer to current vertex.
				const DPOINT2D* operator-> (
					) const { return (&**this); }

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					) {
					++m_Index;
					return (*this);
					}

				//! Post-increment operator.
				CONST_ITERATOR operator++(int
					) {
					CONST_ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				CONST_ITERATOR& operator-- (
					) {
					--m_Index;
					return (*this);
					}

				//! Post-decrement operator.
				CONST_ITERATOR operator--(int
					) {
					CONST_ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				//! Equality operator.
				bool operator== (
					const CONST_ITERATOR& rhs
					) const { return (m_Index == rhs.m_Index && m_PolyLine == rhs.m_PolyLine); }

				//! Inequality operator.
				bool operator!= (
					const CONST_ITERATOR& rhs
					) const { return (!(*this == rhs)); }

				//! Get index of current vertex.
				INT32 GetIndex (
					) const { return (m_Index); }

			protected:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				INT32 m_Index;
				const POLYLINE* m_PolyLine;
				#endif
			};

		//! Non-const iterator.
		class ITERATOR {
			public:

				//! Default constructor
				ITERATOR (
					) :
					m_Index(0),
					m_PolyLine(0)
					{ }

				//! Internal constructor
				ITERATOR (
					INT32 index,
					POLYLINE* PolyLine
					) : 
					m_Index(index),
					m_PolyLine(PolyLine)
					{ }

				//! Copy constructor
				ITERATOR (
					const ITERATOR& rhs
					) : 
					m_Index(rhs.m_Index),
					m_PolyLine(rhs.m_PolyLine)
					{ }

				//! Assignment.
				ITERATOR& operator= (
					const ITERATOR& rhs
					) {
					if (this != &rhs) {
						m_Index = rhs.m_Index;
						m_PolyLine = rhs.m_PolyLine;
						}
					return (*this);
					}
					
				//! Dereference operator, returns reference to current vertex.
				DPOINT2D& operator* (
					) const { return (m_PolyLine->GetVertexRef(m_Index)); }

				//! Dereference operator, returns pointer to current vertex.
				DPOINT2D* operator-> (
					) const { return (&**this); }

				//! Pre-increment operator.
				ITERATOR& operator++ (
					) {
					m_Index ++;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator++(int
					) {
					ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				ITERATOR& operator-- (
					) {
					m_Index --;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator--(int
					) {
					ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				//! Equality operator
				bool operator== (
					const ITERATOR& rhs
					) const { return (m_Index == rhs.m_Index && m_PolyLine == rhs.m_PolyLine); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

			protected:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				INT32 m_Index;
				POLYLINE *m_PolyLine;
				friend class CONST_ITERATOR;
				#endif
			};

		class SECTION {
			public:
				SECTION (
					const DRECT2D& Extents,
					INT32 StartVertex,
					INT32 EndVertex
					) : 
					m_Extents(Extents),
					m_VertexRange(StartVertex, EndVertex)
					{ }
					
				~SECTION (
					) { }
					
				const DRECT2D& GetExtents (
					) const { return (m_Extents); }
					
				const INT32_RANGE& GetVertexRange (
					) const { return (m_VertexRange); }
					
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DRECT2D m_Extents;
				INT32_RANGE m_VertexRange;

				friend class POLYLINE;
				#endif
			};
		typedef DOUBLE_ARRAY<SECTION> SECTIONARRAY;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class CLIP;
		class INTERSECTINFO;
		class SECTIONS;
		class SPLIT;
		class STRIPS;
		#endif
		
		//! Default constructor, creates empty unclosed 2D polyline.
		POLYLINE ();

		//! Construct with specified attributes.
		explicit POLYLINE (
			bool closed,								//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0,						//!< Initial Z value for POLYLINE if NumDim != 3
			UINT32 ReservePoints = 1				//!< Initial number of points to reserve in buffer
			);

		//! Constructor from DRECT2D, sets closed flag.
		//! Sets "Extents" optimization
		explicit POLYLINE (
			const DRECT2D& rect,						//!< DRECT2D to create a line from
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DRECT3D, sets closed flag.
		//! Sets "Extents" optimization
		explicit POLYLINE (
			const DRECT3D& rect						//!< DRECT3D to create a line from
			);
			
		//! Constructor from legacy DPOLYGON.
		explicit POLYLINE (
			const DPOLYGON& dpoly,					//!< Legacy DPOLYGON to create a line from
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DPOINT2D array and number of points.
		POLYLINE (
			const DPOINT2D* pts,
			INT32 NumPoints,
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DOUBLE_ARRAY<double>.
		//! This constructor transfers ownership from the DOUBLE_ARRAY<>
		explicit POLYLINE (
			DOUBLE_ARRAY<double>& rhs,
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DOUBLE_ARRAY<DPOINT2D>.
		explicit POLYLINE (
			const DOUBLE_ARRAY<DPOINT2D>& rhs,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DOUBLE_ARRAY<DPOINT2D>.
		explicit POLYLINE (
			const SIMPLE_ARRAY<LPOINT2D>& rhs,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from an array of LPOINT2D.
		explicit POLYLINE (
			const LPOINT2D* points,
			INT32 NumPoints,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
			
		//! Constructor from DOUBLE_ARRAY<DPOINT3D>
		explicit POLYLINE (
			const DOUBLE_ARRAY<DPOINT3D>& rhs,
			bool closed = false						//!< Create and maintain a closed polyline
			);
			
		//! Copy constructor
		//! Will use any optimizations associated with "rhs"
		POLYLINE (
			const POLYLINE& rhs
			);

		//! Section copy constructor
		//! NO optimizations associated with "rhs" will be used
		POLYLINE (
			const POLYLINE& rhs, 					//!< POLYLINE to copy from
			INT32 StartVertex, 						//!< Start vertex of 'rhs
			INT32 EndVertex							//!< End vertex of 'rhs' (Inclusive)
			);
		
		//! Destructor		
		~POLYLINE (
			);

		//! Assignment
		//! Will keep any optimizations associated with "rhs"
		//! If POLYLINE::Reserved() was called on 'this' to pre-allocate the point buffer, do not call this method,
		//! call POLYLINE::operator+=() or POLYLINE::Append().
		POLYLINE& operator= (
			const POLYLINE& rhs
			);

		//! Append 'rhs' to 'this'
		//! If both 'rhs' and 'this' have "Extents" optimization, this method update 'this' Extents optimization
		POLYLINE& operator+= (
			const POLYLINE& rhs
			);

		//! Add a vertex to the line
		ERRVALUE AddVertex (
			const DPOINT2D& point,					//!< Point to add
			INT32 index = -1							//!< Default location is the end of the line
			);

		//! Add a vertex to the line
		ERRVALUE AddVertex (
			const DPOINT3D& point,					//!< Point to add
			INT32 index = -1							//!< Default location is the end of the line
			);
			
		//! Append a POLYLINE to this one, by default removing duplicates and collinear points at the joined section
		//! This method will update the "Extents" optimization for 'this' if both 'this' and 'rhs' have that optimization
		ERRVALUE Append (
			const POLYLINE& rhs,
			bool RemovePoints = true
			);
			
		// Apply a z scale factor to each of the z coordinate values if 'this' has them, otherwise apply it to the z value stored
		void ApplyZScale (
			double zscale								//!< Z Scale factor to apply to z coordinate (pt[i].z *= zscale)
			);
			
		//! Assign point values from legacy DPOLYGON structure
		ERRVALUE Assign (
			const DPOLYGON& dpoly,
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);

		//! Assign point values from DPOINT2D array and number of points
		ERRVALUE Assign (
			const DPOINT2D* pts,
			INT32 NumPoints,
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
					
		//! Assign point values from SIMPLE_ARRAY<LPOINT2D> buffer
		ERRVALUE Assign (
			const SIMPLE_ARRAY<LPOINT2D>& rhs,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);

		//! Assign point values from SIMPLE_ARRAY<LPOINT2D> buffer
		ERRVALUE Assign (
			const LPOINT2D* rhs,
			int NumPoints,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
					
		//! Assign point values from DOUBLE_ARRAY<DPOINT2D> buffer
		ERRVALUE Assign (
			const DOUBLE_ARRAY<DPOINT2D>& rhs,
			bool closed = false,						//!< Create and maintain a closed polyline
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);
					
		//! Assign point values from DOUBLE_ARRAY<DPOINT3D> buffer
		ERRVALUE Assign (
			const DOUBLE_ARRAY<DPOINT3D>& rhs,
			bool closed = false						//!< Create and maintain a closed polyline
			);
			
		//! Assign point values from a DRECT2D
		ERRVALUE Assign (
			const DRECT2D& rect,						//!< DRECT2D to crete a line from
			double ZValue = 0.0						//!< Initial Z value for POLYLINE
			);
			
		//! Assign point values from a DRECT3D
		ERRVALUE Assign (
			const DRECT3D& rect						//!< DRECT2D to crete a line from
			);
			
		//! Assign polyline from XMLNODE.
		ERRVALUE AssignFromXML (
			const XMLNODE *topnode
			);

		//! Attach a MIDOUBLEARRAY instance to 'this' by taking ownership of the MIDOUBLEARRAY buffer.  After this call, the 
		//! MIDOUBLEARRAY buffer is no longer valid
		void Attach (
			MIDOUBLEARRAY& rhs,
			INT32 NumPoints,
			bool closed = false,						//!< Create and maintain a closed polyline
			DIMENSION dimension = DIMENSION_2D,	//!< Create a 2D or 3D line
			double ZValue = 0.0						//!< Initial Z value for POLYLINE if NumDim != 3
			);

		//! Return a CONST_ITERATOR that refers to the beginning of the POLYLINE
		CONST_ITERATOR Begin (		  
			) const { return (CONST_ITERATOR(0, this)); }

		//! Return an ITERATOR that refers to the beginning of the POLYLINE
		ITERATOR Begin (					
			) { return (ITERATOR(0, this)); }

		//! Clear POLYLINE of point data	
		void Clear (
			);

		//! Clear POLYLINE of all optimizations
		void ClearOptimizations (
			);

		//! Clear POLYLINE of point data	
		void ClearPoints (
			INT32 StartPosn = 0,
			INT32 EndPosn = INT32_MAX				//!< Inclusive
			);
			
		//! Clip a polyline to a rectangle, call delegate for each segment.
		//! @return TRUE if part of the line is inside the clipping area, FALSE if not,  < 0 error.
		int Clip (
			const DRECT2D& ClipRect,								//!< Rectangle to clip to
			DELEGATE_ERRVALUE_POLYLINE DelegateOnClip,		//!< Delegate to call for each clipped polyline
			CLIPMODE ClipMode = CLIPMODE_Inside
			);

		//! Clip a polyline to a rectangle, call the ClipTarget for each segment.
		//! @return TRUE if part of the line is inside the clipping area, FALSE if not,  < 0 error.
		int Clip (
			const DRECT2D& ClipRect,								//!< Rectangle to clip to
			CLIP& ClipTarget,
			CLIPMODE ClipMode = CLIPMODE_Inside					//!< Clipping mode
			);

		//! Compare the two lines and return the result of the comparison.
		//! Uses "Extent" and "Section" optimizations
		//! @return COMPRESULT depending on the relationship of 'OtherLine' to 'this'
		COMPRESULT CompareLineToLine (
			const POLYLINE& OtherLine
			) const;

		//! Compare the line against the polygon and return the result of the comparison.
		//! Uses "Extent", "Section" and "Strip" optimizations
		//! @return COMPRESULT depending on the relationship of 'Polygon' to 'this'
		COMPRESULT CompareLineToPolygon (
			const POLYLINE& Polygon
			) const;

		//! Compare the line against the polygon and return the result of the comparison.
		//! Uses "Extent", "Section" and "Strip" optimizations
		//! @return COMPRESULT depending on the relationship of 'Polygon' to 'this'
		COMPRESULT CompareLineToPolygon (
			const DRECT2D& rect
			) const;

		//! Compare the two polygons and return the result of the comparison.
		//! Uses "Extent", "Section" and "Strip" optimizations
		//! @return COMPRESULT depending on the relationship of 'OtherPoly' to 'this'
		COMPRESULT ComparePolygonToPolygon (
			const POLYLINE& OtherPoly
			) const;

		//! Compute the angle for the line using the sum of all the angles in the line
		//! @return Angle summation of the angles between segments
		double ComputeAngle (
			) const;

		//! Find area of polygon if the line is closed
		//! This method assumes that the polygon is topologically correct (i.e. it does not cross itself)
		//! @return: Area of the polygon if a polygon, Zero otherwize
		double ComputeArea (
			) const {
			double area;
			ComputeStats(&area, 0);
			return (area);
			}
			
		//! Compute an buffer from the line using a buffer distance and a bufferside.
		ERRVALUE ComputeBuffer (
			const double BufferDistance,			
			POLYLINELIST& BufferLineList,
			const BUFFERSIDE BufferSide = BUFFERSIDE_Both
			) const;

		//! Compute 2D centroid of polygon or line.
		//! This method assumes that the polygon is topologically correct (i.e. it does not cross itself)
		void ComputeCentroid (
			DPOINT2D& cent
			) const;
			
		//! Compute 3D centroid.
		//! If closed, X/Y values will be same as for 2D and Z will be center of Z range.
		void ComputeCentroid (
			DPOINT3D& cent
			) const;
			
		//! Compute the shortest distance between the two lines.  If the lines cross, return zero.
		//! Uses "Section" optimization
		//! @return The shortest distance between the two lines
		double ComputeDistance (
			const POLYLINE& OtherLine,
			bool ExtentsOverlap = true			//!< Set to false if the extents of the two lines DO NOT overlap.  If set to 'false' this method does not need to check for segment intersections
			) const;

		//! Determine a point on the line given a distance from the start or end of the line
		//! @return: True if point found on the line, false if not
		bool ComputeDistPointOnLine (
			double Distance,
			DPOINT3D& NewPoint,					//!< Computed point based on distance RETURNED
			bool DistanceFromStart = true
			) const;
			
		//! Determine a point on the line given a distance from the start or end of the line
		//! @return Vertex index closest to the point returned, based on direction searched, or -1 if not found.
		INT32 ComputeDistPointAndVertexOnLine (
			double Distance,
			DPOINT3D& NewPoint,					//!< Computed point based on distance RETURNED
			bool DistanceFromStart = true
			) const;
			
		//! Determine the extents of the line
		//! Uses and / or sets the "Extent" optimization
		void ComputeExtents (
			DRECT3D& rect
			) const;
		
		//! Determine the extents of a sub-section of the line
		void ComputeExtents (
			DRECT3D& rect,
			INT32 StartVertex,
			INT32 EndVertex				//!< Inclusive
			) const;
			
		//! Determine the extents of the line
		//! Uses and / or sets the "Extent" optimization
		void ComputeExtents (
			DRECT2D& rect
			) const;
		
		//! Determine the extents of a sub-section of the line
		void ComputeExtents (
			DRECT2D& rect,
			INT32 StartVertex,
			INT32 EndVertex				//!< Inclusive
			) const;

		//! Compute point on interior of polygon or on line if not closed.
		void ComputeInteriorPoint (
			DPOINT3D& pt					//!< Point returned
			) const;
		
		//! Determine length of the line
		//! @return: Length of the line in line units
		double ComputeLength (
			) const;
			
		//! Determine length of the line to the point specified
		//! @return: Length of the line in line units to the point specified
		double ComputeLength (
			const DPOINT3D& point				//!< Compute distance from start of line to point given
			) const;
			
		//! Determine length of the longest segment of the line
		//! @return: Length of the longest segment of the line in line units
		double ComputeLongestSegmentLength (
			) const;

		//! Determine maximum length between two vertices on the polygon
		//! Uses the "Section" and "Extent" optimizations
		//! @return Maximum dimension of the polygon			
		double ComputeMaxDimension (
			) const;
			
		//! Determine line mid-point
		void ComputeMidPoint (
			DPOINT3D& MidPoint					//!< Computed mid-point RETURNED
			) const;

		//! Compute an offset from the line using a distance and a join type.		
		//! Positive buffer distances will result in a buffer to the right of the line
		//! assuming Cartesian coordinates (+ right/up, - left/down)
		ERRVALUE ComputeOffset (
			double OffsetDistance,
			POLYLINE& OffsetLine,
			JOINTYPE JoinType = JOINTYPE_Miter
			) const;
			
		//! Compute an offset from the line using a start distance, end distance and a join type.
		//! Positive buffer distances will result in a buffer to the right of the line
		//! assuming Cartesian coordinates (+ right/up, - left/down)
		ERRVALUE ComputeOffset (
			double StartOffsetDistance,
			double EndOffsetDistance,
			POLYLINE& OffsetLine,
			JOINTYPE JoinType = JOINTYPE_Miter
			) const;
			
		//! Compute an offset from the line using a start distance, end distance and a join type.
		//! Positive buffer distances will result in a buffer to the right of the line
		//! assuming Cartesian coordinates (+ right/up, - left/down)
		ERRVALUE ComputeOffset (
			const SIMPLE_ARRAY<double>& OffsetDistance,			//!< One entry for each vertex in the line, sign of each distance needs to be the same
			POLYLINE& OffsetLine,
			JOINTYPE JoinType = JOINTYPE_Miter
			) const;
			
		//! Find orientation of polygon if the line is closed
		//! This method assumes that the polygon is topologically correct (i.e. it does not cross itself)
		//! @return: Orientation of the polygon, ORIENTATION_Unknown otherwize
		ORIENTATION ComputeOrientation (
			) const { return (ComputeStats(0, 0)); }
			
		//! Determine a point on the line given a percentage from the start or end of the line
		//! @return Vertex index closest to the point returned, based on direction searched.
		INT32 ComputePercentPointOnLine (
			double Percent,						//!< Value between 0.0 and 1.0
			DPOINT3D& NewPoint,					//!< Computed point based on distance RETURNED
			bool DistanceFromStart = true
			) const;
			
		//! Compute statistics for the polygon
		//! This method assumes that the polygon is topologically correct (i.e. it does not cross itself)
		//! @return: Orientation of the polygon, ORIENTATION_Unknown otherwize
		ORIENTATION ComputeStats (
			double *const area,						//!< Area value RETURNED, NULL if not needed
			DPOINT2D *const centroid				//!< Centroid value RETURNED, NULL if not needed
			) const;
			
		//! Conflate a POLYLINE to itself with the changed vertex priority assigned to the set.
		//! If there is a change to be made to get the line to match along a set of segments, 
		//! the higher number vertices will be picked to change.
		//! Uses the "Section" optimization
		ERRVALUE Conflate (
			double thresh,
			bool& LineChanged
			);
		
		//! Conflate a set of POLYLINE's with the changed vertex priority assigned to the set.
		//! If there is a change to be made to get the lines to match along a set of segments, the 'LineList' will
		//! be picked to change its points before 'this' will.
		//! Uses the "Section" and "Extent" optimizations
		ERRVALUE Conflate (
			POLYLINELIST& LineList,
			double thresh,
			bool& LineChanged,
			BITSET_UNOWNED& LineListSet			//!< Matches the 'LineList' and tells which lines have been changed
			);
		
		//! Convert point coordinates using affine transformation		
		void ConvertForward (
			const TRANS2D_AFFINE& taf				//!< Transformation to apply
			);
		
		//! Convert point coordinates using generic transformation		
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN& tmg				//!< Transformation to apply
			);

		//! Convert point coordinates using 3D perspective transformation		
		void ConvertForward (
			const TRANS3D& t3d						//!< Transformation to apply
			);
		
		//! Convert point coordinates using COORDTRANS.
		ERRVALUE ConvertForward (
			const SPATREF::COORDTRANS& trans		//!< Transformation to apply
			);

		//! Convert point coordinates using generic transformation with densification
		ERRVALUE ConvertForwardDense (
			const TRANS2D_MAPGEN& tmg				//!< Transformation to apply
			);

		//! Convert point coordinates using COORDTRANS with densification
		ERRVALUE ConvertForwardDense (
			const SPATREF::COORDTRANS& trans,	//!< Transformation to apply
			double tolerance							//!< Densification tolerance in resulting CRS
			);

		//! Convert point coordinates using affine transformation		
		void ConvertInverse (
			const TRANS2D_AFFINE& taf				//!< Transformation to apply
			);
		
		//! Convert point coordinates using generic transformation		
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN& tmg				//!< Transformation to apply
			);

		//! Convert point coordinates using generic transformation with densification
		ERRVALUE ConvertInverseDense (
			const TRANS2D_MAPGEN& tmg				//!< Transformation to apply
			);

		//! Convert point coordinates using 3D perspective transformation		
		void ConvertInverse (
			const TRANS3D& t3d						//!< Transformation to apply
			);
		
		//! Convert point coordinates using COORDTRANS.
		ERRVALUE ConvertReverse (
			const SPATREF::COORDTRANS& trans		//!< Transformation to apply
			);

		//! Convert point coordinates using COORDTRANS with densification
		ERRVALUE ConvertReverseDense (
			const SPATREF::COORDTRANS& trans,	//!< Transformation to apply
			double tolerance							//!< Densification tolerance in resulting CRS
			);

		//! Convert polygon from double's to INT32's (no rounding), removes colinear points.
		ERRVALUE ConvertToLong (
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Convert polygon from double's to INT32's (no rounding), removes colinear points.
		//! @return Number of points placed in 'PointArray'.
		INT32 ConvertToLong (
			LPOINT2D *PointArray
			) const;

		//! Convert polygon from double's to INT32's (floor method), removes colinear points.
		ERRVALUE ConvertToLongFloor (
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Convert polygon from double's to INT32's (floor method), removes colinear points.
		//! @return Number of points placed in 'PointArray'.
		INT32 ConvertToLongFloor (
			LPOINT2D *PointArray
			) const;

		//! Convert polygon from double's to INT32's with scaling and offset (floor method), removes colinear points.
		//! @return Number of points placed in 'PointArray'.
		INT32 ConvertToLongFloor (
			LPOINT2D *PointArray,			//!< Point array to contain result
			const DPOINT2D& scale,			//!< Scaling factors
			const DPOINT2D& offset			//!< Offset values added after scaling
			) const;

		//! Convert polygon from double's to INT32's (NearInt truncating), removes colinear points.
		ERRVALUE ConvertToLongNearInt (
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Convert polygon from double's to INT32's (NearInt truncating), removes colinear points.
		//! @return Number of points placed in 'PointArray'.
		INT32 ConvertToLongNearInt (
			LPOINT2D *PointArray
			) const;

		//! Convert polygon from double's to INT32's with rounding, removes colinear points.
		ERRVALUE ConvertToLongRound (
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Convert polygon from double's to INT32's with rounding, removes colinear points.
		//! @return Number of points placed in 'PointArray'.
		INT32 ConvertToLongRound (
			LPOINT2D *PointArray
			) const;

		//! Convert polygon from double's to INT32's with rounding, keep colinear points.
		ERRVALUE ConvertToLongRoundKeepDups (
			SIMPLE_ARRAY<LPOINT2D>& PointArray
			) const;

		//! Delete a line vertex
		//! If the line is a closed polygon and removing a vertex causes less than 4 vertices
		//! to remain, the line is opened
		void DeleteVertex (						//! Remove a vertex from the line
			int index								//!< Index of vertex to remove
			);
			
		//! Detach the MIDOUBLEARRAY from 'this' to 'rhs'.  'this' does not have control of the buffer after this call
		void Detach (
			MIDOUBLEARRAY& rhs
			);

		//! Detach the MIDOUBLEARRAY from 'this' to 'rhs'.  'this' is not usable until Attach() is called, if you try, your process will die.
		void DetachBuffer (
			MIDOUBLEARRAY& rhs
			);

		//! Dump coordinates to string.
		//! Though intended for debugging, not for serialization, the format is compatible
		//! with AssignFromXML when maximum precision is used.
		void DumpCoords (
			MISTRING& result,							//!< String to append to
			int indent,									//!< Amount to indent each coordinate
			bool maxprec = false						//!< Use maximum precision
			) const;

		//! Return an ITERATOR that refers to the end of the POLYLINE, not a valid point
		ITERATOR End (						
			) { return (ITERATOR(m_NumPoints, this)); }

		//! Return a CONST_ITERATOR that refers to the end of the POLYLINE, not a valid point
		CONST_ITERATOR End (				
			) const { return (CONST_ITERATOR(m_NumPoints, this)); }

		//! Exchange two polylines.
		//! All iterators associated with either polyline will become invalid.
		void Exchange (
			POLYLINE& other							//!< Other polyline to exchange with
			);

		//! Exclusive Union (XOR) operator polygon with 'this'
		//! 'this' and 'PolyLineList' must be conflated using POLYLINE::Conflate()
		//! "PolyLineList" will contain ORIENTATION_Clockwise for shell polygons and ORIENTATION_CounterClockwise for islands
		ERRVALUE ExclusiveUnion (
			POLYLINE& OperPoly,
			POLYLINELIST& PolyLineList,			//!< Results of operation returned here
			COMBINERESULT& CombineResult
			);
			
		//! Extract a portion of the line into a new line based on a basepoint and distance from that basepoint
		void Extract (
			POLYLINE& NewLine,						//!< Result of extraction
			const DPOINT3D& BasePoint,				//!< Point to extract from based on "ExtractPoint"
			double Distance,
			EXTRACT ExtractPoint,
			bool ExtrudePastEnds = false
			) const;
			
		//! Extract a portion of the line into a new line based on two point on the line
		void Extract (
			POLYLINE& NewLine,						//!< Result of extraction
			const DPOINT3D& StartPoint,			//!< Point to start extraction from
			const DPOINT3D& EndPoint				//!< Point to end extraction from
			) const;
			
		//! Extract a portion of the line into a new line based on position and number of points
		void Extract (
			INT32 StartPosn,
			INT32 NumPoints,
			POLYLINE& ExtractLine
			) const;
		
		//! Find closest point on the line (not necessarily at a vertex) to specified point.
		//! @return: Distance to closest point on the line, not necessarily a vertex point
		double FindClosestPoint (
			const DPOINT2D& point,	 		  		//!< Point to check
			DPOINT2D& retpt			  				//!< Point on line found RETURNED
			) const;
		
		//! Find closest point on the line (not necessarily at a vertex) to specified point.
		//! This method uses linear interpolation to determine the correct z value.
		//! @return: Distance to closest point on the line, not necessarily a vertex point
		double FindClosestPoint (
			const DPOINT3D& point,	 		  		//!< Point to check
			DPOINT3D& retpt			  				//!< Point on line found RETURNED
			) const;

		//! Find closest point on the line (not necessarily at a vertex) to specified point.
		//! This method uses linear interpolation to determine the correct z value.
		//! This method is faster than the above 'FindClosestPoint' methods because it will 
		//! 	make less calls to DistPointLineSegD().
		//! @return: Distance to closest point on the line, may not be smaller than 'MaxmimumDistance'
		double FindClosestPoint (
			const DPOINT3D& point,	 		  		//!< Point to check
			double MaximumDistance,					//!< Maximum distance allowed between point on the line and 'point'
			DPOINT3D& retpt,			  				//!< Point on line found RETURNED
			INT32& Vertex								//!< Index of vertex BEFORE the location of the point RETURNED
			) const;

		//! Find closest segment to specified point.
		//! @return Index of first vertex of segment, may be last polyline vertex if closed.
		int FindClosestSegment (
			const DPOINT2D& point,					//!< Point to check
			double *distance = 0						//!< Distance returned, 0 if don't need
			) const;
			
		//! Find closest vertex to specified 2D point.
		//! @return: Distance to closest vertex on the line.
		double FindClosestVertex (
			const DPOINT2D& point,	 		  		//!< Point to check
			INT32& index					  			//!< Index of closest vertex returned
			) const;
		
		//! Find closest vertex to specified 2D point.
		//! @return: Index of closest vertex.
		int FindClosestVertex (
			const DPOINT2D& point,	 		  		//!< Point to check
			double *distance = 0						//!< Distance returned, 0 if don't need
			) const;
		
		//! Find closest vertex to specified 3D point.
		//! This method uses linear interpolation to determine the Z value.
		//! @return: Distance to closest vertex on the line
		double FindClosestVertex (
			const DPOINT3D& point,	 		  		//!< Point to check
			INT32& index						  		//!< Index of closest vertex returned
			) const;

		//! Generate the "Extent" optimization.
		//! Used mostly for Compare...() methods and to setup the "Strip" optimization
		void GenerateExtentOptimization (
			DRECT2D& Extents
			) const;

		//! Generate the "Extent" optimization.
		//! Used mostly for Compare...() methods and to setup the "Stript" optimization
		void GenerateExtentOptimization (
			) const;

		//! Generate the "Section" optimization.
		//! Used for many methods.  A "Section" is defined as piece of the source line (defined by "NumVertices")
		//! and that sections extents.
		ERRVALUE GenerateSectionOptimization (
			SECTIONS*& Sections,									//!< Section information RETURNED, you must free pointer
			INT32 NumVertices = 64
			) const;

		//! Generate the "Section" optimization.
		//! Used for many methods.  A "Section" is defined as piece of the source line (defined by "NumVertices")
		//! and that sections extents.  
		void GenerateSectionOptimization (
			INT32 NumVertices = 64
			) const;

		//! Generate the "Strip" optimization.
		//! Used for IsPointInside() method.  A "Strip" is defined as a list of segments that fall within a range of y values.
		ERRVALUE GenerateStripOptimization (
			STRIPS*& Strips,										//!< Strip information RETURNED, you must free pointer
			INT32 NumDivs = 0
			) const;

		//! Generate the "Strip" optimization.
		//! Used for IsPointInside() method.  A "Strip" is defined as a list of segments that fall within a range of y values.
		ERRVALUE GenerateStripOptimization (
			INT32 NumDivs = 0
			) const;

		//! Get current line dimension
		DIMENSION GetDimension (
			) const { return ((m_NumDim == 2) ? DIMENSION_2D : DIMENSION_3D); }

		//! Get the extents of the POLYLINE if known, this method will NOT compute the extents.
		//! @return Pointer to extents or NULL if not known.
		const DRECT2D* GetExtentsIfKnown () const;

		//! Get the number of points in the line
		//! @return: Number of points in the line
		INT32 GetNumPoints (
			) const { return (m_NumPoints); }

		#ifndef GENERATING_DOXYGEN_OUTPUT
		double* GetPointer (
			int index = 0
			) { return (m_Points + GetOffset(index)); }
			
		const double* GetPointer (
			int index = 0
			) const { return (m_Points + GetOffset(index)); }
		#endif // GENERATING_DOXYGEN_OUTPUT
		
		//! Get Test Point
		//! Obtains valid test point for the line, either the middle vertex or. 
		//! if a two point line, the midpoint
		void GetTestPoint (
			DPOINT3D& pt
			) const;
			
		//! Get 2D vertex.
		//! @return: Constant reference to a 2D point
		const DPOINT2D& GetVertex (
			INT32 index
			) const { return (*reinterpret_cast<const DPOINT2D*>(GetPointer(index))); }

		//! Get a 3D point 
		void GetVertex3D (
			INT32 index,
			DPOINT3D& pt
			) const {
			const double *p = GetPointer(index);
			pt.x = p[0];
			pt.y = p[1];
			pt.z = (m_NumDim == 2) ? m_ZValue : p[2];
			return;
			}

		//! Get 'X' coordinate of a vertex.
		//! @return: 'X' coordinate
		double GetVertexX (
			INT32 index
			) const { return (GetPointer(index)[0]); }

		//! Get 'Y' coordinate of a vertex.
		//! @return: 'Y' coordinate
		double GetVertexY (
			INT32 index
			) const { return (GetPointer(index)[1]); }

		//! Get 'Z' coordinate of a vertex.
		//! @return: 'Z' coordinate
		double GetVertexZ (
			INT32 index
			) const { return ((m_NumDim == 2) ? (m_ZValue) : GetPointer(index)[2]); }

		//! Get vertices into DOUBLE_ARRAY<DPOINT2D> array.
		ERRVALUE GetVertices (
			DOUBLE_ARRAY<DPOINT2D>& vertices			//!< Vertices returned
			) const;
			
		//! Get vertices into DOUBLE_ARRAY<DPOINT3D> array.
		ERRVALUE GetVertices (
			DOUBLE_ARRAY<DPOINT3D>& vertices			//!< Vertices returned
			) const;
			
		//! Get vertices into DPOLYGON
		ERRVALUE GetVertices (
			DPOLYGON& dpoly
			) const;

		//! Get XML representation.
		XMLNODE* GetXML (
			XMLNODE *parentnode,						//!< Parent node
			const char *nodename = 0				//!< Node name, default will be "Polygon" or "Line" depending on whether closed or not
			) const;
			
		//! Get polyline Z value, only useful for 2D lines
		double GetZValue (
			) const { return (m_ZValue); }

		//! Does 'this' have strip optimization
		bool HasSectionOptimization (
			) const;

		//! Does 'this' have strip optimization
		bool HasStripOptimization (
			) const;

		//! Intersect operator polygon with 'this'
		//! "PolyLineList" will contain ORIENTATION_Clockwise for shell polygons and ORIENTATION_CounterClockwise for islands
		//! 'this' and 'PolyLineList' must be conflated using POLYLINE::Conflate()
		//! @return '1' if "PolylineList" has one entry and is the same as 'this', 0 for all other cases, < 0 error
		int Intersect (
			POLYLINE& OperPoly,
			POLYLINELIST& PolyLineList,			//!< Results of operation returned here
			COMBINERESULT& CombineResult
			);
			
		//! Remove redundant and overlapping intersection entries from INTERSECTINFO.		
		void IntersectClean (
			INTERSECTINFO& IntsInfo
			) const;
			
		//! Search for intersections against itself.  Intersection information is stored in the INTERSECTINFO
		//! Uses the "Section" and "Extent" optimizations
		ERRVALUE IntersectSearch (
			INTERSECTINFO& IntsInfo,
			double threshold
			) const;
			
		//! Search for intersections against another line.  Intersection information is stored in the INTERSECTINFO instance.  
		//! If information already exists in the INTERSECTINFO, any new intersection information will be appended.
		//! Uses the "Section" and "Extent" optimizations
		ERRVALUE IntersectSearch (
			INTERSECTINFO& IntsInfo,
			const POLYLINE& OtherLine,
			INTERSECTINFO* OtherIntsInfo,						//!< Optional, used if you want to keep intersection info about "OtherLine'
			double threshold
			) const;
			
		//! Split the line according to the INTERSECTINFO information.
		ERRVALUE IntersectSplitLine (
			INTERSECTINFO& IntsInfo,
			SPLIT& SplitInfo,										//!< Callback base class for line splitting
			double threshold,
			bool CallBadSegment = false						//!< Set to true if an intersection causes invalid lines to be generated
			) const;
			
		//! Check if the line is marked closed
		//! @return: True if the line is marked closed
		bool IsClosed (
			) const { return (m_LineClosed); }

		//! Determine if the two lines are exactly equal 
		//! This method uses "Exact" point comparisons
		//! @return 'True' if they are, 'False' if not
		bool IsEqual (
			const POLYLINE& rhs
			) const;

		//! Determine if the two lines are equivalent 
		//! This method uses "Fuzzy" point comparisons and the direction of the lines are not relevant
		//! @return 'True' if they are, 'False' if not
		bool IsEquivalentTo (
			const POLYLINE& rhs,
			double threshold = 0.0
			) const;

		//! Check if the point is inside the polygon
		//! Uses the "Strip" optimization if exists
		//! @return: True if the point is in, False if not or the line is not closed
		bool IsPointInside (
			const DPOINT2D& pt						//!< Point to check
			) const;

		//! Returns true if the POLYLINE describes a simple, unrotated rectangle
		bool IsRectangle (
			) const;
		
		//! Move a vertex by some dx,dy
		//! If FixedVertex1 and/or FixedVertex2 are specified, all points
		//! between the vertex being moved and the fixed verticies will also
		//! be moved by an ammount inversely proportional to their distance
		//! from the vertex being moved.
		//! If you specify two fixed verticies, one had better be less than
		//! index and one greater than index or you will get weird results.
		//! Also, specifying a FixedVertex which is just index +/- 1 is the
		//! same as not specifying it.
		void MoveVertex (
			int index,					//!< Vertex to move
			const DPOINT2D& delta,	//!< Move distance
			int FixedVertex1 = -1,	//!< Vertex to keep in place
			int FixedVertex2 = -1	//!< Vertex to keep in place
			);

		//! Remove colinear and duplicate vertices along the line.
		//! @return True if line changed, false if not
		bool RemoveColinearVertices (
			double thresh = 0.0,						//!< Threshold value to test for same point or colinear point
			bool KeepEndPoints = false
			);

		//! Remove duplicate and colinear points along the line
		//! \deprecated Use RemoveColinearVertices instead.
		void RemoveDuplicates (
			double thresh = 0.0,						//!< Threshold value to test for same point or colinear point
			bool KeepEndPoints = false
			);

		//! Remove duplicate vertices along the line.
		//! @return True if line changed, false if not
		bool RemoveDuplicateVertices (
			double thresh = 0.0						//!< Threshold value to test for same point
			);

		//! Replace a section of 'this' with a section of 'RepLine'.
		ERRVALUE Replace (
			INT32 StartVertex,
			INT32 EndVertex,						//!< Inclusive
			const POLYLINE& RepLine,
			INT32 RepLineStartVertex = 0,
			INT32 RepLineEndVertex = INT32_MAX
			);

		//! Reserve space for given number of points		
		//! The POLYLINE::operator=() will not use the reserved buffer, 
		//! call POLYLINE::Clear() then POLYLINE::operator+=() or POLYLINE::Append() to use the reserved buffer.
		ERRVALUE Reserve (
			INT32 NumPoints
			);
			
		//! Reverse the vertices of the line
		//! This method will retain the "Extents" and "Orientation" optimizations
		void Reverse ();

		//! Round X/Y values to specified interval.
		//! Note that polyline may have self-intersections after rounding.
		void Round2D (
			double interval,						//!< Rounding interval
			ROUNDMODE mode							//!< ROUNDMODE_Nearest, ROUNDMODE_Up, ROUNDMODE_Down
			);

		//! Close or open the line
		//! Will add closure point if set to 'true'
		void SetClosed (
			bool closed
			);
			
		//! Set line dimension from 2D->3D or 3D->2D				
		ERRVALUE SetDimension (
			DIMENSION dimension
			);
			
		//! Set polygon orientation
		//! Uses the "Orientation" optimization
		void SetOrientation (
			ORIENTATION Orientation
			);

		//! Set the precision of the coordinates in the POLYLINE
		//! 'Precision' is the number of decimal places to keep, '0' for precision of whole numbers, < '0' for the left of the decimal point,
		//! > '0' for right of the decimal point
		void SetPrecision (
			int Precision
			);
			
		//! Setup the "Extent" optimization.
		//! Used mostly for Compare...() methods and to setup the "Strip" optimization
		void SetupExtentOptimization (
			const DRECT2D& Extents
			) const;

		//! Setup the "Orientation" optimization.
		//! Used mostly for SetOrientation(), Intersect(), Union(), Subtract(), and ExclusiveUnion() methods
		void SetupOrientationOptimization (
			ORIENTATION Orientation
			) const;

		//! Setup the "Section" optimization.
		//! Used for many methods.  A "Section" is defined as piece of the source line (defined by "NumVertices")
		//! and that sections extents.  
		void SetupSectionOptimization (
			const SECTIONS& Sections						//!< Sections from GenerateSectionOptimization()
			) const;

		//! Setup the "Strip" optimization.
		//! Used for IsPointInside() method.  A "Strip" is defined as a list of segments that fall within a range of y values.
		void SetupStripOptimization (
			const STRIPS& Strips
			) const;

		//! Set a vertex on the line to a specified 2D point
		void SetVertex (
			INT32 index,								//!< Vertex to set
			const DPOINT2D& pt						//!< Point to set vertex to
			);

		//! Set a vertex on the line to a specified 3D point
		void SetVertex (
			INT32 index,								//!< Vertex to set
			const DPOINT3D& pt						//!< Point to set vertex to
			);
			
		//! Set X coordinate for specified vertex.
		void SetVertexX (
			INT32 index,								//!< Vertex to set
			double x										//!< 'X' coordinate to set vertex to
			);

		//! Set Y coordinate for specified vertex.
		void SetVertexY (
			INT32 index,								//!< Vertex to set
			double y										//!< 'Y' coordinate to set vertex to
			);

		//! Set Z coordinate for specified vertex.
		void SetVertexZ (
			INT32 index,								//!< Vertex to set
			double z										//!< 'Z' coordinate to set vertex to
			);

		//! Set polyline Z value, only useful for 2D lines
		void SetZValue (
			double ZValue
			) {m_ZValue = ZValue; }

		//! Spline the line according to the SPLINE
		void Spline (
			SPLINE SplineType,
			INT32 NumKnots,								//!< Number of points between points (Bezier - Number of points in the line)
			double tolerance,
			SPLINEFLAGS flags = SPLINEFLAG_None
			);

		//! Spline the line using (distance, z) ordering according to the SPLINE.  Used to spline the z values of a line.
		void SplineZ (
			SPLINE SplineType,
			INT32 NumKnots,								//!< Number of points between points (Bezier - Number of points in the line)
			double tolerance,
			SPLINEFLAGS flags = SPLINEFLAG_None
			);

		//! Take the first and last points in the line and make a two point line from them
		void Straighten (
			);

		//! Subtract operator polygon from 'this'
		//! "PolyLineList" will contain ORIENTATION_Clockwise for shell polygons and ORIENTATION_CounterClockwise for islands
		//! 'this' and 'PolyLineList' must be conflated using POLYLINE::Conflate()
		//! @return '1' if "PolylineList" has one entry and is the same as 'this', 0 for all other cases, < 0 error
		int Subtract (
			POLYLINE& OperPoly,
			POLYLINELIST& PolyLineList,			//!< Results of operation returned here
			COMBINERESULT& CombineResult
			);
			
		//! Thin the line points based on method and factor		
		void Thin (
			THINMETHOD Method,
			double Factor
			);
			
		//! Transfer polyline data ownership from the DOUBLE_ARRAY<> to itself
		void TransferOwnerFrom (
			DOUBLE_ARRAY<DPOINT2D>& rhs			//!< Point list to transfer ownership from
			);
			
		//! Transfer polyline data ownership from the DOUBLE_ARRAY<> to itself
		void TransferOwnerFrom (
			DOUBLE_ARRAY<DPOINT3D>& rhs			//!< Point list to transfer ownership from
			);
			
		//! Transfer polyline data ownership from itself to the DOUBLE_ARRAY<>
		void TransferOwnerTo (
			DOUBLE_ARRAY<DPOINT2D>& rhs			//!< Point list to transfer ownership to
			);
			
		//! Transfer polyline data ownership from itself to the DOUBLE_ARRAY<>
		void TransferOwnerTo (
			DOUBLE_ARRAY<DPOINT3D>& rhs			//!< Point list to transfer ownership to
			);
			
		//! Union (ADD) operator polygon to 'this'
		//! "PolyLineList" will contain ORIENTATION_Clockwise for shell polygons and ORIENTATION_CounterClockwise for islands
		//! 'this' and 'PolyLineList' must be conflated using POLYLINE::Conflate()
		//! @return '1' if "PolylineList" has one entry and is the same as 'this', 0 for all other cases, < 0 error
		int Union (
			POLYLINE& OperPoly,
			POLYLINELIST& PolyLineList,			//!< Results of operation returned here
			COMBINERESULT& CombineResult
			);
			
		//! Validate that the polygon does not intersect itself.
		//! @return TRUE if the polygon was valid, FALSE if not, < 0 error
		int ValidatePolygon (
			) const;
			
		//! Validate that the polygon does not intersect itself, resolve intersections if needed and call delegate for each resulting validated polygon.
		//! @return TRUE if the polygon was already valid, FALSE if new polygons were generated, < 0 error
		int ValidatePolygon (
			DELEGATE_ERRVALUE_POLYLINE DelegateOnValidated	//!< Delegate to call for each validated polygon, will be called for original polygon if already valid
			) const;
			
		//! Validate that the polygon does not intersect itself, resolve intersections if needed and call ClipTarget for each resulting validated polygon.
		//! @return TRUE if the polygon was already valid, FALSE if new polygons were generated, < 0 error
		int ValidatePolygon (
			CLIP& ClipTarget						//!< Target for resulting validated polygon, will be called for original polygon if already valid
			) const;
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		POLYLINE (const POLYLINE& rhs, const SECTION& Section);
		
		void AddJoinPoints (const DPOINT2D& BasePt, double OffsetDistance, const DPOINT3D& op1, const DPOINT3D& op2, JOINTYPE JoinType);
		ERRVALUE AddVertexNoCopy (const DPOINT3D& point, INT32 index);
		//! Combine- Combine a polygon to another polygon.  Both CW, intersection. Both CCW, union.
		//!		Source CCW and Operator CW, Source - Operator.  Source CW and Operator CCW, Operator - Source.
		//! "PolyLineList" will contain ORIENTATION_Clockwise for shell polygons and ORIENTATION_CounterClockwise for islands
		//! @return '1' if "PolylineList" has one entry and is the same as 'this', 0 for all other cases, < 0 error
		int Combine (const POLYLINE& OperPoly, POLYLINELIST& PolyLineList, COMBINERESULT& CombineResult) const;		//!< Not completely defined yet
		void ComputeExtentsRaw (DRECT2D& rect) const {ComputeExtentsRaw(rect, 0, m_NumPoints-1); }
		void ComputeExtentsRaw (DRECT2D& rect, INT32 StartVertex, INT32 EndVertex) const;
		double ComputeLengthRaw () const;
			
		void ComputeSegment (
			INT32 Index1,
			INT32 Index2,
			INT32 Index3,
			INT32 Index4,
			int NumKnots,
			double Tolerance,
			bool UseMinDist,
			const DOUBLE_ARRAY<double>& T,
			POLYLINE& TempLine
			) const;
		
		void ComputeSegmentQ (
			INT32 Index1,
			INT32 Index2,
			INT32 Index3,
			int NumKnots,
			double Tolerance,
			bool UseMinDist,
			const DOUBLE_ARRAY<double>& T,
			POLYLINE& TempLine
			) const;
		
		bool ComputeSegmentOffset (INT32 PointNum, double StartOffsetDistance, double EndOffsetDistance, DPOINT3D& op0, DPOINT3D& op1) const;
		ORIENTATION ComputeStatsRaw (double& area, DPOINT2D& centroid) const;

		struct INTPOINT;			
		ERRVALUE FindIntersections (const POLYLINE& ClipPoly, DOUBLE_ARRAY<INTPOINT>& IntPoints, bool& HasCommonVertex, bool& HasCommonSegment, MISTRING *pStrDebug = 0) const;
		void FixStartEndPoints (const DPOINT2D& Start, const DPOINT2D& End, double Tolerance);
		INT32 GetOffset (int index) const { return ((m_StartOffset + index) * m_NumDim); }
		//! Used by non-const iterator.	
		DPOINT2D& GetVertexRef (INT32 index) { return (*reinterpret_cast<DPOINT2D*>(GetPointer(index))); }
			
		ERRVALUE IntersectSearchRaw (INTERSECTINFO& ThisIntsInfo, const POLYLINE& OtherLine, INTERSECTINFO *pOtherIntsInfo, double threshold, bool ReturnOnTrueCross, bool IgnoreLineEndPoints) const;

		ERRVALUE MakeCopy ();	
		void MoveVertexSub(int index, const DPOINT2D& delta, int fixed);
		ERRVALUE Resize (INT32 NewSize);
		void ResizeExc (INT32 NewSize);
		ERRVALUE ResizeNoCopy (INT32 NewSize);
		void SetVertexNoCopy (INT32 index, const DPOINT3D& pt);
			
		static bool TestPoint (const POLYLINE& LHS, const POLYLINE& RHS, const double *lhs, const double *rhs, double threshold);
		static bool TestDistance (const POLYLINE& P, const POLYLINE& Q, const double *p1, const double *p2, const double *q1, double threshold);
		void ValidateClosure ();

		bool m_LineClosed;				//!< Is the line closed
		double m_ZValue;					//!< Z value for 3DXY lines
		int m_NumDim;						//!< Number of dimensions in the line (2 or 3)
		INT32 m_NumPoints;				//!< Number of points in the line
		INT32 m_StartOffset;				//!< Offset to start of represented points
		double *m_Points;
		
		class PRIV;
		PRIV *m_Priv;
		
		class OPTIMIZE;
		friend class OPTIMIZE;
		mutable OPTIMIZE *m_Optimize;
		
		static const UINT32 s_AllocInc;		// Amount to increase buffer by
		static const double s_AngleStep;		// Amount increment angle by
		static const double s_MiterLimit;	// Maximum miter angle limit

		friend class POLYLINE::ITERATOR;
		friend class POLYLINE::STRIPS;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


inline bool operator== (
	const POLYLINE& lhs,
	const POLYLINE& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const POLYLINE& lhs,
	const POLYLINE& rhs
	) { return (!lhs.IsEqual(rhs)); }

inline bool operator< (			// for POLYLINELIST to compile
	const POLYLINE& lhs,
	const POLYLINE& rhs
	) { return (lhs.GetNumPoints() < rhs.GetNumPoints()); }


class GEOMLIBCLASSEXPORT POLYLINELIST : public MILIST<POLYLINE> { };

//! Clip() base callback class
class POLYLINE::CLIP {
	public:
		virtual ERRVALUE OnClip (
			const POLYLINE& PolyLine
			) = 0;
		ERRVALUE DoClip (
			const POLYLINE& PolyLine
			) { return (OnClip(PolyLine)); }
	};
	

#ifndef GENERATING_DOXYGEN_OUTPUT

GEOMLIBEXPORT void IntersectDumpInfo (const POLYLINE::INTERSECTINFO& ints, MISTRING& LogString, INT32 LineNum);

#endif // GENERATING_DOXYGEN_OUTPUT

//! Intersection information container class
class POLYLINE::INTERSECTINFO {
	public:
	
		struct CROSSPT {
			enum FLAGS {
				FLAG_KnowIsCross =	0x0001,		//!< Know whether is actual crossing point (for non-type 1 intersections)
				FLAG_IsCross =			0x0002,		//!< Indicates if actual crossing point, only valid if FLAG_KnowIfCross set
				};
			DPOINT3D m_point;
			INT32 m_index;
			UINT16 m_flags;
			UINT8 m_type;				//!< Intersection type (1-9)
			UINT8 m_spare;
			
			CROSSPT (
				) { memset(this, 0, sizeof(*this)); }
			};

		struct OVERLAP {
			DPOINT2D m_start;
			DPOINT2D m_end;
			INT32 m_index;
			#ifndef GENERATING_DOXYGEN_OUTPUT
			INT32 m_spare;			// For alignment
			#endif // GENERATING_DOXYGEN_OUTPUT

			OVERLAP (
				) { memset(this, 0, sizeof(*this)); }
			};
			
		INTERSECTINFO (
			) : m_IsCleaned(true) { }

		void Clean (
			const POLYLINE& PolyLine
			);

		void Clear (
			) {
			m_CrossPts.Clear();
			m_Overlaps.Clear();
			m_IsCleaned = true;
			return;
			}
			
		void DumpInfo (
			MISTRING& LogString,
			INT32 LineNum								//!< Line number associated with "ints"
			) const { IntersectDumpInfo (*this, LogString, LineNum); }
			
		const SIMPLE_ARRAY<CROSSPT>& GetCrossPoints (
			) const { return (m_CrossPts); }
	
		SIMPLE_ARRAY<CROSSPT>& GetCrossPoints (
			) { return (m_CrossPts); }
			
		INT32 GetNumCrossPoints (
			) const { return (m_CrossPts.GetNumItems()); }
	
		INT32 GetNumOverlaps (
			) const { return (m_Overlaps.GetNumItems()); }
	
		const SIMPLE_ARRAY<OVERLAP>& GetOverlaps (
			) const { return (m_Overlaps); }
	
		SIMPLE_ARRAY<OVERLAP>& GetOverlaps (
			) { return (m_Overlaps); }
			
		bool HasOverlaps (
			) const { return (m_Overlaps.GetNumItems() != 0); }
	
		bool IsCleaned (
			) const { return (m_IsCleaned); }
			
		bool IsEmpty (
			) const { return (m_CrossPts.GetNumItems() == 0); }
	
		void SetCleaned (
			bool value = true
			) {m_IsCleaned = value; }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<CROSSPT> m_CrossPts;
		SIMPLE_ARRAY<OVERLAP> m_Overlaps;
		bool m_IsCleaned;	

		static int CompareCrossPt (
			CROSSPT *p1,
			CROSSPT *p2,
			void *vpPolyLine
			);

		static int CompareOverlap (
			OVERLAP *o1,
			OVERLAP *o2,
			void *vpPolyLine
			);

		void CleanCrossPts (
			const POLYLINE& PolyLine
			);
			
		void CleanOverlaps (
			const POLYLINE& PolyLine
			);
			
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! POLYLINE section information to improve performance of certain algorithms such as POLYLINE::Conflate();
class GEOMLIBCLASSEXPORT POLYLINE::SECTIONS {
	public:
		SECTIONS (
			);

		SECTIONS (
			const SECTIONS& rhs
			);

		~SECTIONS (
			);
			
		SECTIONS& operator= (
			const SECTIONS& rhs
			);

		//! Clear Section information			
		void Clear (
			);

		//! Create the sections given the POLYLINE and possibly the number of vertices to use in each section
		ERRVALUE ComputeSections (
			const POLYLINE& PolyLine,		//!< POLYLINE to compute strips for
			INT32 NumVertices = 64
			);
			
		const POLYLINE::SECTIONARRAY& GetSectionArray (
			) const;
			
		//! Has the section list been computed?
		bool IsValid (
			) const;

		void SetSectionArray (
			const POLYLINE::SECTIONARRAY& SectionArray
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_Priv;
		POLYLINE::SECTIONARRAY m_Empty;
		
		POLYLINE::SECTIONARRAY& GetSectionArrayRaw ();
		void GetUniqueSections (SECTIONS& NewSection) const;
		ERRVALUE ComputeSectionsRaw (const POLYLINE& PolyLine, INT32 NumVertices = 64);
		friend class POLYLINE;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
	
//! IntersectSplitLine() base callback class
class POLYLINE::SPLIT {
	public:
		SPLIT (){ }
		virtual ~SPLIT (){ }

		ERRVALUE OnOverlap (
			const POLYLINE& PolyLine
			) { return (v_OnOverlap(PolyLine)); }

		virtual ERRVALUE OnAdd (
			const POLYLINE& PolyLine
			) = 0;

		//! Check to see if on a bad segment.
		//! @return To keep the line if only segment > 0, 0 to possibly remove it, < 0 error
		virtual int OnBadSegment (
			const POLYLINE&					//!< Reference to polyline
			) { return (0); }

		virtual ERRVALUE OnDelete (
			) { return (0); }

	private:
		//! Called when another line overlaps part of the line to be split.
		//! The overlapped section is returned in 'PolyLine'
		virtual ERRVALUE v_OnOverlap (
			const POLYLINE&					//!< Reference to polyline
			) { return (0); }
	};
		

//! POLYLINE strip information to improve performance of certain algorithms such as POLYLINE::IsPointInside()
class GEOMLIBCLASSEXPORT POLYLINE::STRIPS {
	public:

		STRIPS (
			);

		STRIPS (
			const STRIPS& rhs
			);

		~STRIPS (
			);
			
		STRIPS& operator= (
			const STRIPS& rhs
			);

		//! Clear Strip information			
		void Clear (
			);

		//! Create the strip list given the POLYLINE, the extents of the POLYLINE, 
		//! and possibly the number of strips (divisions) to use
		ERRVALUE ComputeStrips (
			const POLYLINE& PolyLine,		//!< POLYLINE to compute strips for
			const DRECT2D& PolyExtents,
			INT32 NumDivs = 0					//!< Default is # of points / 100
			);
			
		//! Create the strip list given the POLYLINE
		//! and possibly the number of strips (divisions) to use
		ERRVALUE ComputeStrips (
			const POLYLINE& PolyLine,		//!< POLYLINE to compute strips for
			INT32 NumDivs = 0					//!< Default is # of points / 100
			);
			
		//! Get a strip cooresponding to the y coordinate of the point
		const SIMPLE_ARRAY<INT32_RANGE>& GetStrip (
			double y
			) const;

		//! Get a strip cooresponding to the y range
		ERRVALUE GetStrip (
			double yinit,
			double ylast,
			SIMPLE_ARRAY<INT32_RANGE>& Strip
			) const;

		//! Has the strip list been computed?			
		bool IsValid (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_Priv;
		SIMPLE_ARRAY<INT32_RANGE> m_Empty;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(POLYLINE::SPLINEFLAGS)
#endif

#undef GEOMLIBEXPORT
#undef GEOMLIBCLASSEXPORT
	
#endif		// INC_MI32_POLYLINE_H
