/**
 *	\file mi32/polynomial.h
 *	\brief POLYNOMIAL class definitions
 *
 *	\if NODOC
 *	$Id: polynomial.h_v 1.3 2006/01/25 14:42:29 mju Exp $
 *
 *	$Log: polynomial.h_v $
 *	Revision 1.3  2006/01/25 14:42:29  mju
 *	Don't pragma warning 4251 as disabled in stddefns.
 *
 *	Revision 1.2  2004/09/02 15:43:29  mju
 *	Add docs.
 *
 *	Revision 1.1  2004/09/02 15:33:52  mju
 *	Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_POLYNOMIAL_H
#define	INC_MI32_POLYNOMIAL_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifdef GEOMDLL
	#define CLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSEXPORT MI_DLLCLASSIMPORT
#endif

//=====================================================================================================================
//! 1-dimensional simple polynomial.
//! Polynomial is of the form:
//! 	c[0] + c[1]*x + c[2]*x^2 + ... + c[order]*x^order.
//! After construction must call Initialize() or SetCoefficients() before using other methods.
class CLASSEXPORT POLYNOMIAL_1D {
	public:

		//! Default constructor.
		POLYNOMIAL_1D (
			);

		//! Destructor
		~POLYNOMIAL_1D (
			);

		//! Add sample for polynomial computation.
		ERRVALUE AddSample (
			double x,						//!< Sample position
			double value					//!< Sample value
			);

		//! Compute polynomial fitted to previously added samples.
		ERRVALUE Compute (
			);

		//! Evaluate polynomial at specified position.
		double Evaluate (
			double x							//!< Position to evaluate at
			) const;

		//! Get polynomial coefficients.
		//! Coefficient index is same as power for position when evaluating.
		//! Eg. value = c[0] + c[1]*x + c[2]*x^2 + ... + c[order]*x^order.
		const DOUBLE_ARRAY<double>& GetCoefficients (
			) const { return (m_Coeff); }

		//! Initialize
		ERRVALUE Initialize (
			unsigned order					//!< Polynomial order, maximum of 15
			);

		//! Directly set polynomial coefficients.
		ERRVALUE SetCoefficients (
			unsigned order,				//!< Polynomial order, maximum of 15
			const double *Coeff			//!< Coefficient array of size (order + 1)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_Order;
		DOUBLE_ARRAY<double> m_Coeff;
		DOUBLE_ARRAY<double> m_A;
		DOUBLE_ARRAY<double> m_b;
		#endif
	};


//=====================================================================================================================
//! 2-dimensional simple polynomial.
//! Polynomial is of the form:
//!	c0 + c1*x + c2*y + c3*x^2 + c4*x*y + c5*y^2 + c6*x^3 + c7*x^2*y + c8*x*y^2 + c9*y^3 + ...
//! After construction must call Initialize() or SetCoefficients() before using other methods.
class CLASSEXPORT POLYNOMIAL_2D {
	public:

		//! Default constructor.
		POLYNOMIAL_2D (
			);

		//! Destructor
		~POLYNOMIAL_2D (
			);

		//! Add sample for polynomial computation.
		ERRVALUE AddSample (
			double x,						//!< Sample X position
			double y,						//!< Sample Y position
			double value					//!< Sample value
			);

		//! Add sample for polynomial computation.
		ERRVALUE AddSample (
			const DPOINT2D& pos,			//!< Sample position
			double value					//!< Sample value
			) { return (AddSample(pos.x,pos.y,value)); }

		//! Compute polynomial fitted to previously added samples.
		ERRVALUE Compute (
			);

		//! Evaluate polynomial at specified position.
		double Evaluate (
			double x,						//! X position
			double y							//! Y position
			) const;

		//! Evaluate polynomial at specified position.
		double Evaluate (
			const DPOINT2D& pos			//!< Position
			) const { return (Evaluate(pos.x,pos.y)); }

		//! Get polynomial coefficients.
		//! Coefficients are sequenced by increasing sum of X and Y powers.
		//! Eg. value = c0 + [c1*x + c2*y] + [c3*x^2 + c4*x*y + c5*y^2] + [c6*x^3 + c7*x^2*y + c8*x*y^2 + c9*y^3] + ...
		const DOUBLE_ARRAY<double>& GetCoefficients (
			) const { return (m_Coeff); }

		//! Initialize
		ERRVALUE Initialize (
			unsigned order					//!< Polynomial order, maximum of 10
			);

		//! Directly set polynomial coefficients.
		ERRVALUE SetCoefficients (
			unsigned order,				//!< Polynomial order, maximum of 10
			const double *Coeff			//!< Coefficient array of size (order + 1) * (order + 2) / 2
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		int m_Order;
		DOUBLE_ARRAY<double> m_Coeff;
		DOUBLE_ARRAY<double> m_A;
		DOUBLE_ARRAY<double> m_b;
		#endif
	};


//=====================================================================================================================

#undef	CLASSEXPORT

#endif	// INC_MI32_POLYNOMIAL_H
