/**
 * \file mi32/trend.h
 *
 * \if NODOC
 * $Id: trend.h_v 1.36 2012/09/18 13:52:19 mju Exp $
 *
 * $Log: trend.h_v $
 * Revision 1.36  2012/09/18 13:52:19  mju
 * Dox ctor fix.
 *
 * Revision 1.35  2004/09/03 18:14:50  mju
 * Create TREND2D class, remove legacy definitions.
 *
 * Revision 1.24  1999/01/05  19:12:57  scowan
 *
 * \endif
****************************************************************************/

#ifndef	INC_MI32_TREND_H
#define	INC_MI32_TREND_H

#ifndef	INC_MI32_POLYNOMIAL_H
#include <mi32/polynomial.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJITEM;
	class RASTER;
	class NULLMASK;
	}
#endif


//! 2-dimensional polynomial trend.
class TREND2D {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class GENERATOR;
		#endif

		//! Parameters for computing trend.
		class PARM {
			public:

				PARM (
					) : m_Order(1), m_BorderCells(0), m_SampleCols(1), m_SampleRows(1), m_PreserveRange(false) { }

				void Set (
					int Order,					//!< Polynomial order, usually 1 or 2
					int BorderCells,			//!< Number of border cells to use, 0 to use entire raster
					int SampleCols,			//!< Sample interval for columns (>= 1)
					int SampleRows,			//!< Sample interval for rows (>= 1)
					bool PreserveRange		//!< Preserve original dynamic range when compute global scale and offset
					) { m_Order = Order; m_BorderCells = BorderCells; m_SampleCols = SampleCols; m_SampleRows = SampleRows; m_PreserveRange = PreserveRange; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				int m_Order;					// Polynomial order, usually 1 or 2
				int m_BorderCells;			// Number of border cells to use, 0 to use entire raster
				int m_SampleCols;				// Sample interval for columns (>= 1)
				int m_SampleRows;				// Sample interval for rows (>= 1)
				bool m_PreserveRange;		// Preserve original dynamic range when compute global scale and offset
				friend class TREND2D;
				friend class TREND2D::GENERATOR;
				#endif
			};

		//! Constructor.
		TREND2D ();

		//! Destructor.
		~TREND2D ();

		//! Add trend to value at specified coordinates.
		double AddTrend (
			double x,
			double y,
			double value
			) const;

		//! Compute trend using open raster and optional mask.
		//! Mask must be binary and same size as raster.
		ERRVALUE Compute (
			const PARM& parm,								//!< Parameters to compute from
			RVC::RASTER& ObjRaster,						//!< Open raster
			RVC::RASTER *pObjMask = 0					//!< Optional open mask
			);

		//! Compute trend using raster and optional mask specified by OBJITEM.
		ERRVALUE Compute (
			const PARM& parm,								//!< Parameters to compute from
			const RVC::OBJITEM& ObjItemRaster,		//!< Raster
			const RVC::OBJITEM *pObjItemMask = 0	//!< Optional mask
			);

		//! Get global offset for computed trend.
		double GetGlobalOffset (
			) const { return (m_GlobalOffset); }

		//! Get global scale for computed trend.
		double GetGlobalScale (
			) const { return (m_GlobalScale); }

		//! Get parameters used in last trend computation.
		const PARM& GetParm (
			) const { return (m_Parm); }

		//! Get raw trend value at specified coordinates.
		double GetValue (
			double x,
			double y
			) const { return (m_Polynomial.Evaluate(x,y)); }

		//! Subtract trend to value at specified coordinates.
		double SubtractTrend (
			double x,
			double y,
			double value
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PARM m_Parm;
		POLYNOMIAL_2D m_Polynomial;
		double m_GlobalScale;
		double m_GlobalOffset;
		friend class TREND2D::GENERATOR;
		#endif
	};


#endif	// INC_MI32_TREND_H 
