/**
 * \file rvc/spattree.h 
 * \brief interface for the RVC::SPATIALTREE class.
 *
 * \if NODOC
 * $Id: spattree.h_v 1.20 2011/12/23 14:04:48 mju Exp $
 *
 * $Log: spattree.h_v $
 * Revision 1.20  2011/12/23 14:04:48  mju
 * Remove redundant hdr.
 *
 * Revision 1.19  2010/04/22 20:02:52  mju
 * Add search using unsignedset.
 *
 * Revision 1.14  2005/12/09 17:29:57  scowan
 * Updated to use credentials.
 *
 * Revision 1.1  2000/09/29 16:40:07  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_SPATTREE_H
#define INC_RVC_SPATTREE_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef INC_MI32_RANGE_H
	#include <mi32/range.h>
#endif

#ifndef INC_MI32_POINT_H
	#include <mi32/point.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class UNSIGNEDSET;
namespace RVC {
	class SPATIALTREEHANDLE;
	}
#endif


namespace RVC {

//! Manager for RVC::OBJTYPE_SpatialTree objects.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_SpatialTree.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Vector
class SPATIALTREE : public RVC::OBJECT {
	public:

		//! Spatial Tree tier information structure.
		//! Nodes are stored by row, thus X is incremented fastest and Z is incremented slowest.
		struct TIERINFO {
			// No members may ever be added to this structure.
			// The DLL does not support expansion.
		
			DPOINT3D m_Origin;
			DPOINT3D m_NodeSize;
			WPOINT3D m_NumNodes;
			UINT8 spare[2];
		
			TIERINFO (
				) { Clear(); }	

			//! Calculate node index for given 2D point coordinates.
			//! @return Node index or -1 if outside tier extents.
			INT32 CalcNode2D (
				const DPOINT2D& point
				) const {
				INT32 xpos = CalcNodeX(point.x);
				if (xpos < 0 || xpos >= m_NumNodes.x) return (-1);
				INT32 ypos = CalcNodeY(point.y);
				if (ypos < 0 || ypos >= m_NumNodes.y) return (-1);
				return (ypos * m_NumNodes.x + xpos);
				}
		
			//! Calculate node index for given 3D point coordinates.
			//! @return Node index or -1 if outside tier extents.
			INT32 CalcNode3D (
				const DPOINT3D& point
				) const {
				INT32 node = CalcNode2D(point);
				if (m_NumNodes.z > 1 && node >= 0) {
					INT32 zpos = CalcNodeZ(point.z);
					if (zpos < 0 || zpos >= m_NumNodes.z) return (-1);
					node += zpos * m_NumNodes.x * m_NumNodes.y;
					}
				return (node);
				}
		
			//! Calculate node X index for given X coordinate.
			//! @return X index, may be outside valid node range.
			INT32 CalcNodeX (
				double x
				) const { return (static_cast<INT32>(floor((x - m_Origin.x) / m_NodeSize.x))); }
		
			//! Calculate node Y index for given Y coordinate.
			//! @return Y index, may be outside valid node range.
			INT32 CalcNodeY (
				double y
				) const { return (static_cast<INT32>(floor((y - m_Origin.y) / m_NodeSize.y))); }
		
			//! Calculate node Z index for given Z coordinate.
			//! @return Z index, may be outside valid node range.
			INT32 CalcNodeZ (
				double z
				) const { return (static_cast<INT32>(floor((z - m_Origin.z) / m_NodeSize.z))); }

			//! Clear the structure			
			void Clear (
				) {memset(static_cast<void*>(this), 0, sizeof(*this));}
			};

		//! Default constructor
		SPATIALTREE (
			);

		//! Copy constructor
		SPATIALTREE (
			const RVC::SPATIALTREE& rhs
			);

		//! Destructor
		virtual ~SPATIALTREE (
			);

		//! Assignment
		RVC::SPATIALTREE& operator= (
			const RVC::SPATIALTREE& rhs
			);

		//! Search spatial tree for elements which may overlap specified rectangle.
		ERRVALUE Search (
			ELEMTYPE elemtype,							//!< Element type
			const DRECT2D& rect,						//!< Rectangle in spatial tree coordinates to search
			SIMPLE_ARRAY<INT32_RANGE>& elemranges		//!< Array of element ranges returned
			);
		
		//! Search spatial tree for elements which may overlap specified rectangle.
		ERRVALUE Search (
			ELEMTYPE elemtype,						//!< Element type
			const DRECT2D& rect,						//!< Rectangle in spatial tree coordinates to search
			UNSIGNEDSET& ElemSet						//!< Element set returned
			);
		
		//! Get maximum number of node array elements in spatial tree for specified element type.
		//! @return Maximum number of node array elements or error < 0.
		INT32 GetMaxNodeArrayElems (
			ELEMTYPE elemtype							//!< Element type
			);
		
		//! Get number of tiers in spatial tree for specified element type.
		//! @return Number of tiers, 0 if none or error < 0.
		INT32 GetNumTiers (
			ELEMTYPE elemtype							//!< Element type
			);
		
		//! Create new spatial tree object.
		ERRVALUE Make (
			const RVC::OBJECT& parent,					//!< Parent object to create spatial tree object under
			const RVC::DESCRIPTOR& Descriptor,		//!< Name of new spatial tree object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);
		
		//! Create new spatial tree object.
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,							//!< Source of new spatial tree object, default is application name
			const CREDENTIALS* Credentials
#else
			const char* Source = 0,							//!< Source of new spatial tree object, default is application name
			const CREDENTIALS* Credentials = 0
#endif
			);
		
		//! Read tier from spatial tree.
		ERRVALUE ReadTier (
			ELEMTYPE elemtype,							//!< Element type
			int tier,									//!< Tier index
			TIERINFO& tierinfo,							//!< Tier information returned
			INT32 *firstelemarray						//!< Array of "first elements" by node index returned, caller must allocate
			);
		
		//! Write tier to spatial tree.
		ERRVALUE WriteTier (
			ELEMTYPE elemtype,							//!< Element type
			int tier,									//!< Tier index
			TIERINFO& tierinfo,							//!< Tier information to write
			INT32 *firstelemarray						//!< Array of "first elements" by node index, caller must allocate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		SPATIALTREEHANDLE *m_pSpatialTreeHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_SPATTREE_H
