/**
 * \file spatmod/imagebase.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::BASE class definitions
 *
 * \if NODOC
 * $Id: imagebase.h_v 1.21 2011/10/11 13:59:30 mju Exp $
 *
 * $Log: imagebase.h_v $
 * Revision 1.21  2011/10/11 13:59:30  mju
 * Remove unneeded hdr.
 *
 * Revision 1.20  2011/03/31 21:36:03  mju
 * Add logCreate.
 *
 * Revision 1.19  2011/03/30 21:38:43  mju
 * Add v_getTypeName and log support.
 *
 * Revision 1.18  2011/02/03 21:25:09  mju
 * Add getGeoreference using zoom and change virtual to same.
 *
 * Revision 1.17  2008/01/31 16:58:57  mju
 * Add computeGeoreferenceRegion.
 *
 * Revision 1.16  2007/04/09 17:44:25  mju
 * Remove virtual methods to get separate dimension values with zoom.
 *
 * Revision 1.15  2007/04/09 13:44:31  mju
 * Remove separate dimension access methods with zoom parm.
 *
 * Revision 1.14  2007/04/03 14:52:10  mju
 * Use ifdef to exclude deprecated methods.
 *
 * Revision 1.13  2007/02/27 21:57:22  mju
 * Don't default zoom parms for deprecated fns.
 *
 * Revision 1.12  2007/02/27 19:03:26  mju
 * Add getDimensions with zoom and overridable.
 *
 * Revision 1.11  2007/01/04 15:33:02  mju
 * Make setGeoreference public.
 *
 * Revision 1.10  2007/01/02 22:23:35  mju
 * Add attachPixelTable.
 *
 * Revision 1.9  2006/11/01 23:10:42  mju
 * queryIterator.
 *
 * Revision 1.7  2006/07/27 21:48:14  mju
 * Remove capability_access.
 *
 * Revision 1.6  2006/07/14 21:49:28  mju
 * Add capability_Access.
 *
 * Revision 1.5  2006/06/05 13:17:23  mju
 * Use DIMENSIONS class.
 *
 * Revision 1.4  2005/12/16 16:37:27  mju
 * Add versions of getTotalColumns/Rows that return integer if no zoom parm.
 *
 * Revision 1.3  2005/12/16 16:07:08  mju
 * Use georeference class and add support methods for it.
 *
 * Revision 1.2  2005/08/26 15:15:20  mju
 * Use range type for pixel range instead of queryCapability.
 *
 * Revision 1.1  2005/08/16 15:08:42  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEBASE_H
#define	INC_SPATMOD_IMAGEBASE_H

#ifndef	INC_SPATMOD_IMAGEDIMENSIONS_H
#include <spatmod/imagedimensions.h>
#endif

#ifndef	INC_SPATMOD_IMAGEPIXEL_H
#include <spatmod/imagepixel.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_MILOG_H
#include <mi32/milog.h>
#endif

class REGION2D;

namespace SPATMOD {
namespace IMAGE {

class PIXELTABLE;
class SECTION_ITERATOR;

//=====================================================================================================================

//! Base class for image.
//! Contains image properties, provides no methods for reading/writing actual image data.
class BASE {
	public:

		//! Enumeration of basic image capabilities.
		enum CAPABILITY {
			CAPABILITY_Pull,							//!< Can "pull" from image
			CAPABILITY_Push							//!< Can "push" to image
			};

		//! Enumeration of supported zoom abilities.
		enum ZOOMABILITY {
			ZOOMABILITY_Only1X,						//!< Can only access with zoom = 1
			ZOOMABILITY_List,							//!< Supports specific set of zoom values
			ZOOMABILITY_Range							//!< Supports any zoom in range
			};

		//! Constructor
		BASE ();

		//! Destructor.
		virtual ~BASE ();

		//! Compute georeference region.
		//! If image is not georeferenced returned region's CRS will be undefined.
		ERRVALUE ComputeGeoreferenceRegion (
			REGION2D& region							//!< Region in georeference CRS returned
			) const;

		//! Compute tile column for specified image coordinate.
		INT32 ComputeTileColumn (
			double x,									//!< Image coordinate at specified zoom value
			double zoom = 1							//!< Zoom/magnification factor
			) const;

		//! Compute tile row for specified image coordinate.
		INT32 ComputeTileRow (
			double y,									//!< Image coordinate at specified zoom value
			double zoom = 1							//!< Zoom/magnification factor
			) const;

		//! Get dimensions of unzoomed image.
		//! @return Dimensions
		const DIMENSIONS& GetDimensions (
			) const { return (m_Dimensions); }

		//! Get dimensions at specified zoom value.
		//! If zoom is exactly 1, base image dimensions will be returned.
		ERRVALUE GetDimensions (
			DIMENSIONS& dimensions,					//!< Dimensions returned
			double zoom									//!< Zoom/magnification factor
			) const;

		//! Get georeference of unzoomed image.
		//! @return Pointer to georeference or NULL if none.
		const GEOREFERENCE* GetGeoreference (
			) const { return (m_pGeoreference); }

		//! Get georeference at specified zoom value.
		//! If zoom is exactly 1 and has base georeference, that will be returned.
		//! Will return error if image is not georeferenced or if zoom level is not supported.
		ERRVALUE GetGeoreference (
			GEOREFERENCE& georeference,			//!< Georeference returned
			double zoom									//!< Zoom/magnification factor
			) const;

		//! Access log (const).
		const MILOG& GetLog () const
			{ return (m_Log); }

		//! Access log (non-const).
		MILOG& GetLog ()
			{ return (m_Log); }

		//! Get number of columns of tiles "across" the image.
		//! @return Number of columns of tiles.
		INT32 GetNumColumnsOfTiles (
			) const { return (m_Dimensions.GetNumColumnsOfTiles()); }

		//! Get number of columns per tile.
		//! Note that first and last tiles may have different sizes than each other or interior tiles.
		//! If GetNumColumnsOfTiles is less than 3 then will return 0 for TILEPOSITION_Interior.
		//! If GetNumColumnsOfTiles is less than 2 then the "last" tile will have same dimension as "first" tile.
		//! @return Number of columns per tile.
		double GetNumColumnsPerTile (
			TILEPOSITION tileposition				//!< Tile position
			) const { return (m_Dimensions.GetNumColumnsPerTile(tileposition)); }

		//! Get number of rows of tiles "down" the image.
		//! @return Number of rows of tiles.
		INT32 GetNumRowsOfTiles (
			) const { return (m_Dimensions.GetNumRowsOfTiles()); }

		//! Get number of rows per tile.
		//! Note that first and last tiles may have different sizes than each other or interior tiles.
		//! If GetNumRowsOfTiles is less than 3 then will return 0 for TILEPOSITION_Interior.
		//! If GetNumRowsOfTiles is less than 2 then the "last" tile will have same dimension as "first" tile.
		//! @return Number of rows per tile.
		double GetNumRowsPerTile (
			TILEPOSITION tileposition				//!< Tile position
			) const { return (m_Dimensions.GetNumRowsPerTile(tileposition)); }

		//! Get number of samples per pixel.
		//! Same as GetPixelProperties().GetNumSamples().
		UINT16 GetNumSamples (
			) const { return (GetPixelProperties().GetNumSamples()); }

		//! Get pixel lookup table (for images with PIXEL::TYPE_Indexed).
		//! @return Pointer to lookup table or NULL if none.
		const PIXELTABLE* GetPixelLookupTable (
			) const { return (v_GetPixelLookupTable()); }

		//! Get image pixel properties.
		const PIXEL& GetPixelProperties (
			) const { return (m_PixelProperties); }

		//! Get type of pixel range available.
		RANGETYPE GetPixelRangeType (
			) const { return (m_PixelRangeType); }

		//! Get pixel type.
		//! Same as GetPixelProperties().GetPixelType().
		PIXEL::TYPE GetPixelType (
			) const { return (GetPixelProperties().GetPixelType()); }

		//! Get maximum pixel value.
		//! Use GetPixelRangeType() to determine if specific to image.
		const PIXEL& GetPixelValueMax (
			) const { return (m_PixelValueMax); }

		//! Get minimum pixel value.
		//! Use GetPixelRangeType() to determine if specific to image.
		const PIXEL& GetPixelValueMin (
			) const { return (m_PixelValueMin); }

		//! Get total number of columns in image.
		//! @return Total image columns.
		UINT32 GetTotalColumns (
			) const { return (m_Dimensions.GetTotalColumns()); }

		//! Get total number of rows in image.
		//! @return Total image rows.
		UINT32 GetTotalRows (
			) const { return (m_Dimensions.GetTotalRows()); }

		//! Get class type name, typically used for logging.
		const char* GetTypeName () const
			{ return (v_GetTypeName()); }

		//! Get zoom ability information.
		//! The returned array will contain one or more values, the meaning of which depends on
		//! the ZOOMABILITY value returned.  If ZOOMABILITY_Only1X, the returned array should be ignored and
		//! may be empty.  If ZOOMABILITY_List, the returned array should contain a list of all supported
		//! zoom values (including 1.0).  If ZOOMABILITY_Range, the returned array will contain exactly 2
		//! values with the first being the minimum supported zoom and the last being the maximum.  Returned
		//! values in the array should always be positive.
		ZOOMABILITY GetZoomAbility (
			SIMPLE_ARRAY<double>& ZoomsAllowed	//! Zoom values allowed, interpretation depends on return value
			) const { return (v_GetZoomAbility(ZoomsAllowed)); }

		//! Create log for entire pipeline.
		ERRVALUE LogCreate (
			const char *label = 0,							//!< Label for log
			bool OnlyEnds = false,							//!< Only create log for source/target stages
			MILOG::MODE mode = MILOG::MODE_OwnFile		//!< Logging mode, must use MODE_OwnFile or MODE_SharedFile
			);

		//! Query image for specified capability.
		bool QueryCapability (
			CAPABILITY capability
			) const { return (v_QueryCapability(capability)); }

		//! Query support/efficiency for specified iterator.
		//! @return Efficiency between 0 and 1, 0 indicates not supported, 1 indicates most efficient.
		float QueryIterator (
			const SECTION_ITERATOR& iterator		//!< Iterator
			) const;

		//! Query image for support of specified zoom value.
		bool QueryZoom (
			double zoom
			) const;

		//! Set georeference.
		//! May be done after Initialize, but must usually be done before adding more stages to pipeline.
		//! A copy of the specified georeference is retained.
		void SetGeoreference (
			const GEOREFERENCE *pGeoreference	//!< Pointer to georeference, NULL to remove previously set values
			);

	protected:

		PIXELTABLE *m_pPixelLookupTable;			//!< Pointer to pixel lookup table

		//! Attach pixel lookup table (for PIXEL::TYPE_Indexed).
		//! More efficient than SetPixelLookupTable.
		void AttachPixelLookupTable (
			PIXELTABLE& PixelTable					//!< Pixel table to attach, return value will be empty
			);

		//! Set all base dimensions from another image.
		//! Set total and tile dimensions, and georeference.
		void SetAllDimensions (
			const BASE& image							//!< Image to set dimensions from
			);

		//! Set all base properties (except pixel lookup table) from another image.
		//! Set total and tile dimensions, pixel properties, pixel range if defined,
		//! and georeference.
		void SetAllProperties (
			const BASE& image							//!< Image to set properties from
			);

		//! Set dimensions.
		void SetDimensions (
			const DIMENSIONS& Dimensions
			);

		//! Set pixel lookup table (for PIXEL::TYPE_Indexed).
		//! A copy of the specified table is retained.
		//! Note, AttachPixelLookupTable is more efficient.
		void SetPixelLookupTable (
			const PIXELTABLE *pPixelTable			//!< Pointer to lookup table, NULL to remove previously set table
			);

		//! Set pixel properties for image.
		//! Will also reset pixel range to default.
		void SetPixelProperties (
			const PIXEL& PixelProperties
			);

		//! Set pixel properties, including range, from source image.
		void SetPixelProperties (
			const BASE& image							//!< Image to set properties from
			);

		//! Set specific pixel range and type.
		ERRVALUE SetPixelRange (
			const PIXEL& PixelValueMin,
			const PIXEL& PixelValueMax,
			RANGETYPE RangeType
			);

		//! Set pixel range to default for pixel type.
		//! Also set pixel range type to RANGETYPE_Default.
		void SetPixelRangeDefault (
			);

		//! Set tile dimensions.
		ERRVALUE SetTileDimensions (
			double ColumnsPerTile,
			double RowsPerTile,
			double ColumnsPerTileFirst = 0,
			double RowsPerTileFirst = 0,
			double ColumnsPerTileLast = 0,
			double RowsPerTileLast = 0
			);

		//! Set image dimensions.
		//! Will reset tile dimensions to match image dimensions (resulting in "untiled" image).
		ERRVALUE SetTotalDimensions (
			UINT32 TotalColumns,
			UINT32 TotalRows
			);

		// BASE overridables.

		//! Get georeference information.
		//! Default implementation calculates by scaling, which assumes top and left edges are exactly coincident.
		virtual ERRVALUE v_GetGeoreference (
			GEOREFERENCE& georeference,
			double zoom
			) const;

		//! Get pixel lookup table.
		//! Default implementation returns pointer if set and image has PIXEL::TYPE_Indexed, NULL otherwise.
		virtual const PIXELTABLE* v_GetPixelLookupTable (
			) const;

		//! Query image for specified capability.
		virtual bool v_QueryCapability (
			CAPABILITY capability
			) const;

		//! Query support/efficiency for specified stepping order.
		//! @return Efficiency between 0 and 1, 0 indicates not supported, 1 indicates most efficient.
		//! Default implementation returns 1 for STEPORDER_TileSequential, 1 for STEPORDER_FullSequential
		//! if only one column of tiles and 0 for all other cases.
		virtual float v_QueryIterator (
			const SECTION_ITERATOR& iterator		//!< Iterator
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DIMENSIONS m_Dimensions;
		PIXEL m_PixelProperties;
		PIXEL m_PixelValueMin;
		PIXEL m_PixelValueMax;
		RANGETYPE m_PixelRangeType;
		GEOREFERENCE *m_pGeoreference;
		MILOG m_Log;

		#endif	// GENERATING_DOXYGEN_OUTPUT

		// BASE overridables.

		//! Get dimensions at specified zoom value.
		//! Must be overridden if image supports zooming, default implementation returns error.
		virtual ERRVALUE v_GetDimensions (
			DIMENSIONS& dimensions,					//!< Dimensions returned
			double zoom									//!< Zoom/magnification factor
			) const;

		//! Get class type name.
		//! Should always be overridden.  Do not inline.
		virtual const char* v_GetTypeName () const;

		//! Query image for support of specified zoom value.
		//! Default implementation returns true if zoom==1 and false otherwise.
		virtual ZOOMABILITY v_GetZoomAbility (
			SIMPLE_ARRAY<double>& ZoomsAllowed	//! Zoom values allowed, interpretation depends on return value
			) const;

		//! Create log
		virtual ERRVALUE v_LogCreate (
			const char *label,
			bool OnlyEnds,
			MILOG::MODE mode
			) = 0;


	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGEBASE_H
