/**
 * \file spatmod/imagesourcefile.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_FILE definition.
 *
 * \if NODOC
 * $Id: imagesourcefile.h_v 1.4 2011/11/14 21:36:02 mju Exp $
 *
 * $Log: imagesourcefile.h_v $
 * Revision 1.4  2011/11/14 21:36:02  mju
 * Add saveMemBufferToTempFile.
 *
 * Revision 1.3  2011/10/11 14:38:47  mju
 * Remove unneeded hdr.
 *
 * Revision 1.2  2007/02/22 21:13:13  mju
 * Add getMemBuffer methods.
 *
 * Revision 1.1  2007/02/22 21:03:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#define	INC_SPATMOD_IMAGESOURCEFILE_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base for "file" source.
class SOURCE_FILE : public SOURCE {
	public:

		//! Get file path.
		const FILEPATH& GetFilePath (
			) const { return (m_FilePath); }

		//! Get (const) reference to memory buffer.
		const SIMPLE_ARRAY<UINT8>& GetMemBuffer (
			) const { return (m_MemBuffer); }

		//! Attempt to locate and read georeference based on file path.
		//! @return True if successful, false if not.
		bool ReadGeoreference ();

	protected:

		//! Construct with FILEPATH.
		explicit SOURCE_FILE (
			const FILEPATH& FilePath
			);

		//! Construct with memory buffer (SIMPLE_ARRAY).
		//! Will take ownership of buffer.
		explicit SOURCE_FILE (
			SIMPLE_ARRAY<UINT8>& MemBuffer	//!< Memory buffer, will be returned empty
			);

		//! Construct with memory buffer (pointer/size).
		//! Will take ownership of buffer, pointer will be returned as NULL
		SOURCE_FILE (
			UINT8*& MemBuffer,					//!< Memory buffer, will be returned as NULL
			UINT32 BufferSize						//!< Buffer size
			);

		//! Destructor.
		virtual ~SOURCE_FILE ();

		//! Get reference to memory buffer.
		//! Should avoid changing buffer, method is to support 3rd-party implementation not accepting const buffer.
		SIMPLE_ARRAY<UINT8>& GetMemBuffer (
			) { return (m_MemBuffer); }

		//! Save memory buffer to temporary file, with file deletion on destruction.
		//! This is useful for derived classes that require an actual file to read.
		ERRVALUE SaveMemBufferToTempFile (
			const char *extn = "tmp"			//!< File extension to use
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILEPATH m_FilePath;
		SIMPLE_ARRAY<UINT8> m_MemBuffer;
		bool m_DeleteFileOnClose;

		// Unimplemented
		SOURCE_FILE (const SOURCE_FILE&);
		SOURCE_FILE& operator= (const SOURCE_FILE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEFILE_H

